#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
date: 2022/2/14 20:02
desc: 东方财富网-数据中心-特色数据-机构调研
http://data.eastmoney.com/jgdy/
东方财富网-数据中心-特色数据-机构调研-机构调研统计: http://data.eastmoney.com/jgdy/tj.html
东方财富网-数据中心-特色数据-机构调研-机构调研详细: http://data.eastmoney.com/jgdy/xx.html
"""
import pandas as pd
import requests
from tqdm import tqdm


def stock_jgdy_tj_em(date: str = "20220101") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-机构调研-机构调研统计
    http://data.eastmoney.com/jgdy/tj.html
    :param date: 开始时间
    :type date: str
    :return: 机构调研统计
    :rtype: pandas.DataFrame
    """
    url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        'sortColumns': 'NOTICE_DATE,SUM,RECEIVE_START_DATE,SECURITY_CODE',
        'sortTypes': '-1,-1,-1,1',
        'pageSize': '500',
        'pageNumber': '1',
        'reportName': 'RPT_ORG_SURVEYNEW',
        'columns': 'ALL',
        'quoteColumns': 'f2~01~SECURITY_CODE~CLOSE_PRICE,f3~01~SECURITY_CODE~CHANGE_RATE',
        'source': 'WEB',
        'client': 'WEB',
        'filter': f"""(NUMBERNEW="1")(IS_SOURCE="1")(RECEIVE_START_DATE>'{'-'.join([date[:4], date[4:6], date[6:]])}')"""
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json['result']['pages']
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page+1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json['result']['data'])
        big_df = pd.concat([big_df, temp_df])
    big_df.reset_index(inplace=True)
    big_df["index"] = list(range(1, len(big_df) + 1))
    big_df.columns = [
        "序号",
        "_",
        "代码",
        "名称",
        "_",
        "公告日期",
        "接待日期",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "接待地点",
        "_",
        "接待方式",
        "_",
        "接待人员",
        "_",
        "_",
        "_",
        "_",
        "_",
        "接待机构数量",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "最新价",
        "涨跌幅",
    ]
    big_df = big_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "接待机构数量",
            "接待方式",
            "接待人员",
            "接待地点",
            "接待日期",
            "公告日期",
        ]
    ]
    big_df['最新价'] = pd.to_numeric(big_df['最新价'], errors="coerce")
    big_df['涨跌幅'] = pd.to_numeric(big_df['涨跌幅'], errors="coerce")
    big_df['接待机构数量'] = pd.to_numeric(big_df['接待机构数量'], errors="coerce")
    big_df['接待日期'] = pd.to_datetime(big_df['接待日期']).dt.date
    big_df['公告日期'] = pd.to_datetime(big_df['公告日期']).dt.date
    return big_df


def stock_jgdy_detail_em(date: str = "20220101") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-机构调研-机构调研详细
    http://data.eastmoney.com/jgdy/xx.html
    :param date: 开始时间
    :type date: str
    :return: 机构调研详细
    :rtype: pandas.DataFrame
    """
    url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        'sortColumns': 'NOTICE_DATE,RECEIVE_START_DATE,SECURITY_CODE,NUMBERNEW',
        'sortTypes': '-1,-1,1,-1',
        'pageSize': '50000',
        'pageNumber': '1',
        'reportName': 'RPT_ORG_SURVEY',
        'columns': 'SECUCODE,SECURITY_CODE,SECURITY_NAME_ABBR,NOTICE_DATE,RECEIVE_START_DATE,RECEIVE_OBJECT,RECEIVE_PLACE,RECEIVE_WAY_EXPLAIN,INVESTIGATORS,RECEPTIONIST,ORG_TYPE',
        'quoteColumns': 'f2~01~SECURITY_CODE~CLOSE_PRICE,f3~01~SECURITY_CODE~CHANGE_RATE',
        'source': 'WEB',
        'client': 'WEB',
        'filter': f"""(IS_SOURCE="1")(RECEIVE_START_DATE>'{'-'.join([date[:4], date[4:6], date[6:]])}')"""
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json['result']['pages']
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page+1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json['result']['data'])
        big_df = pd.concat([big_df, temp_df])
    big_df.reset_index(inplace=True)
    big_df["index"] = list(range(1, len(big_df) + 1))
    big_df.columns = [
        "序号",
        "_",
        "代码",
        "名称",
        "公告日期",
        "调研日期",
        "调研机构",
        "接待地点",
        "接待方式",
        "调研人员",
        "接待人员",
        "机构类型",
        "最新价",
        "涨跌幅",
    ]
    big_df = big_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "调研机构",
            "机构类型",
            "调研人员",
            "接待方式",
            "接待人员",
            "接待地点",
            "调研日期",
            "公告日期",
        ]
    ]
    big_df['最新价'] = pd.to_numeric(big_df['最新价'], errors="coerce")
    big_df['涨跌幅'] = pd.to_numeric(big_df['涨跌幅'], errors="coerce")
    big_df['调研日期'] = pd.to_datetime(big_df['调研日期']).dt.date
    big_df['公告日期'] = pd.to_datetime(big_df['公告日期']).dt.date
    return big_df


if __name__ == "__main__":
    stock_jgdy_tj_em_df = stock_jgdy_tj_em(date="20180928")
    print(stock_jgdy_tj_em_df)

    stock_jgdy_detail_em_df = stock_jgdy_detail_em(date="20210915")
    print(stock_jgdy_detail_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/5/18 17:10
Desc: 中国柯桥纺织指数
http://www.kqindex.cn/flzs/jiage
"""
import pandas as pd
import requests
from tqdm import tqdm


def index_kq_fz(symbol: str = "价格指数") -> pd.DataFrame:
    """
    中国柯桥纺织指数
    http://www.kqindex.cn/flzs/jiage
    :param symbol: choice of {'价格指数', '景气指数', '外贸指数'}
    :type symbol: str
    :return: 中国柯桥纺织指数
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "价格指数": "1_1",
        "景气指数": "1_2",
        "外贸指数": "2",
    }
    url = "http://www.kqindex.cn/flzs/table_data"
    params = {
        "category": "0",
        "start": "",
        "end": "",
        "indexType": f"{symbol_map[symbol]}",
        "pageindex": "1",
        "_": "1619871781413",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    page_num = data_json["page"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params = {
            "category": "0",
            "start": "",
            "end": "",
            "indexType": f"{symbol_map[symbol]}",
            "pageindex": page,
            "_": "1619871781413",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"])
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    if symbol == "价格指数":
        big_df.columns = [
            "期次",
            "指数",
            "涨跌幅",
        ]
        big_df['期次'] = pd.to_datetime(big_df['期次'])
        big_df['指数'] = pd.to_numeric(big_df['指数'], errors='coerce')
        big_df['涨跌幅'] = pd.to_numeric(big_df['涨跌幅'], errors='coerce')
    elif symbol == "景气指数":
        big_df.columns = [
            "期次",
            "总景气指数",
            "涨跌幅",
            "流通景气指数",
            "生产景气指数",
        ]
        big_df['总景气指数'] = pd.to_numeric(big_df['总景气指数'], errors='coerce')
        big_df['涨跌幅'] = pd.to_numeric(big_df['涨跌幅'], errors='coerce')
        big_df['流通景气指数'] = pd.to_numeric(big_df['流通景气指数'], errors='coerce')
        big_df['生产景气指数'] = pd.to_numeric(big_df['生产景气指数'], errors='coerce')
    elif symbol == "外贸指数":
        big_df.columns = [
            "期次",
            "价格指数",
            "价格指数-涨跌幅",
            "景气指数",
            "景气指数-涨跌幅",
        ]
        big_df['价格指数'] = pd.to_numeric(big_df['价格指数'], errors='coerce')
        big_df['价格指数-涨跌幅'] = pd.to_numeric(big_df['价格指数-涨跌幅'], errors='coerce')
        big_df['景气指数'] = pd.to_numeric(big_df['景气指数'], errors='coerce')
        big_df['景气指数-涨跌幅'] = pd.to_numeric(big_df['景气指数-涨跌幅'], errors='coerce')
    big_df.sort_values(['期次'], inplace=True, ignore_index=True)
    return big_df


if __name__ == "__main__":
    index_kq_fz_df = index_kq_fz(symbol="价格指数")
    print(index_kq_fz_df)

    index_kq_fz_df = index_kq_fz(symbol="景气指数")
    print(index_kq_fz_df)

    index_kq_fz_df = index_kq_fz(symbol="外贸指数")
    print(index_kq_fz_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/7/6 19:28
Desc: 新浪财经-科创板-实时行情数据和历史行情数据(包含前复权和后复权因子)
"""
import datetime
import re

from byteapp_ak.utils import demjson
import pandas as pd
import requests
from tqdm import tqdm

from byteapp_ak.stock.cons import (
    zh_sina_kcb_stock_payload,
    zh_sina_kcb_stock_url,
    zh_sina_kcb_stock_count_url,
    zh_sina_kcb_stock_hist_url,
    zh_sina_kcb_stock_hfq_url,
    zh_sina_kcb_stock_qfq_url,
    zh_sina_kcb_stock_amount_url,
)


def get_zh_kcb_page_count() -> int:
    """
    所有股票的总页数
    https://vip.stock.finance.sina.com.cn/mkt/#hs_a
    :return: 所有股票的总页数
    :rtype: int
    """
    res = requests.get(zh_sina_kcb_stock_count_url)
    page_count = int(re.findall(re.compile(r"\d+"), res.text)[0]) / 80
    if isinstance(page_count, int):
        return page_count
    else:
        return int(page_count) + 1


def stock_zh_kcb_spot() -> pd.DataFrame:
    """
    新浪财经-科创板实时行情数据, 大量抓取容易封IP
    https://vip.stock.finance.sina.com.cn/mkt/#kcb
    :return: 科创板实时行情数据
    :rtype: pandas.DataFrame
    """
    big_df = pd.DataFrame()
    page_count = get_zh_kcb_page_count()
    zh_sina_stock_payload_copy = zh_sina_kcb_stock_payload.copy()
    for page in tqdm(range(1, page_count + 1), leave=False):
        zh_sina_stock_payload_copy.update({"page": page})
        zh_sina_stock_payload_copy.update({"_s_r_a": "page"})
        res = requests.get(zh_sina_kcb_stock_url, params=zh_sina_stock_payload_copy)
        data_json = demjson.decode(res.text)
        big_df = pd.concat([big_df, pd.DataFrame(data_json)], ignore_index=True)
    big_df.columns = [
        "代码",
        "-",
        "名称",
        "最新价",
        "涨跌额",
        "涨跌幅",
        '买入',
        '卖出',
        '昨收',
        '今开',
        '最高',
        '最低',
        '成交量',
        '成交额',
        '时点',
        '市盈率',
        '市净率',
        '流通市值',
        '总市值',
        '换手率',
    ]
    big_df = big_df[[
        "代码",
        "名称",
        "最新价",
        "涨跌额",
        "涨跌幅",
        '买入',
        '卖出',
        '昨收',
        '今开',
        '最高',
        '最低',
        '成交量',
        '成交额',
        '时点',
        '市盈率',
        '市净率',
        '流通市值',
        '总市值',
        '换手率',
    ]]

    big_df['最新价'] = pd.to_numeric(big_df['最新价'])
    big_df['涨跌额'] = pd.to_numeric(big_df['涨跌额'])
    big_df['涨跌幅'] = pd.to_numeric(big_df['涨跌幅'])
    big_df['买入'] = pd.to_numeric(big_df['买入'])
    big_df['卖出'] = pd.to_numeric(big_df['卖出'])
    big_df['昨收'] = pd.to_numeric(big_df['昨收'])
    big_df['今开'] = pd.to_numeric(big_df['今开'])
    big_df['最高'] = pd.to_numeric(big_df['最高'])
    big_df['最低'] = pd.to_numeric(big_df['最低'])
    big_df['成交量'] = pd.to_numeric(big_df['成交量'])
    big_df['成交额'] = pd.to_numeric(big_df['成交额'])
    big_df['市盈率'] = pd.to_numeric(big_df['市盈率'])
    big_df['市净率'] = pd.to_numeric(big_df['市净率'])
    big_df['流通市值'] = pd.to_numeric(big_df['流通市值'])
    big_df['总市值'] = pd.to_numeric(big_df['总市值'])
    big_df['换手率'] = pd.to_numeric(big_df['换手率'])
    return big_df


def stock_zh_kcb_daily(symbol: str = "sh688399", adjust: str = "") -> pd.DataFrame:
    """
    新浪财经-科创板股票的历史行情数据, 大量抓取容易封IP
    https://finance.sina.com.cn/realstock/company/sh688005/nc.shtml
    :param symbol: 股票代码; 带市场标识的股票代码
    :type symbol: str
    :param adjust: 默认不复权的数据; qfq: 返回前复权后的数据; hfq: 返回后复权后的数据; hfq-factor: 返回后复权因子; hfq-factor: 返回前复权因子
    :type adjust: str
    :return: 科创板股票的历史行情数据
    :rtype: pandas.DataFrame
    """
    res = requests.get(
        zh_sina_kcb_stock_hist_url.format(
            symbol, datetime.datetime.now().strftime("%Y_%m_%d"), symbol
        )
    )
    data_json = demjson.decode(res.text[res.text.find("[") : res.text.rfind("]") + 1])
    data_df = pd.DataFrame(data_json)
    data_df.index = pd.to_datetime(data_df["d"])
    data_df.index.name = "date"
    del data_df["d"]

    r = requests.get(zh_sina_kcb_stock_amount_url.format(symbol, symbol))
    amount_data_json = demjson.decode(r.text[r.text.find("[") : r.text.rfind("]") + 1])
    amount_data_df = pd.DataFrame(amount_data_json)
    amount_data_df.index = pd.to_datetime(amount_data_df.date)
    del amount_data_df["date"]
    temp_df = pd.merge(
        data_df, amount_data_df, left_index=True, right_index=True, how="left"
    )
    try:
        # try for pandas >= 2.1.0
        temp_df.ffill(inplace=True)
    except Exception as e:
        try:
        # try for pandas < 2.1.0          
            temp_df.fillna(method="ffill", inplace=True)
        except Exception as e:
                print("Error:", e)
    temp_df = temp_df.astype(float)
    temp_df["amount"] = temp_df["amount"] * 10000
    temp_df["turnover"] = temp_df["v"] / temp_df["amount"]
    temp_df.columns = [
        "open",
        "high",
        "low",
        "close",
        "volume",
        "after_volume",
        "after_amount",
        "outstanding_share",
        "turnover",
    ]

    if not adjust:
        temp_df.reset_index(inplace=True)
        temp_df['date'] = pd.to_datetime(temp_df['date']).dt.date
        return temp_df

    if adjust == "hfq":
        res = requests.get(zh_sina_kcb_stock_hfq_url.format(symbol))
        hfq_factor_df = pd.DataFrame(
            eval(res.text.split("=")[1].split("\n")[0])["data"]
        )
        hfq_factor_df.columns = ["date", "hfq_factor"]
        hfq_factor_df.index = pd.to_datetime(hfq_factor_df.date)
        del hfq_factor_df["date"]

        temp_df = pd.merge(
            temp_df, hfq_factor_df, left_index=True, right_index=True, how="left"
        )
        try:
            # try for pandas >= 2.1.0
            temp_df.ffill(inplace=True)
        except Exception as e:
            try:
            # try for pandas < 2.1.0          
                temp_df.fillna(method="ffill", inplace=True)
            except Exception as e:
                print("Error:", e)
        temp_df = temp_df.astype(float)
        temp_df["open"] = temp_df["open"] * temp_df["hfq_factor"]
        temp_df["high"] = temp_df["high"] * temp_df["hfq_factor"]
        temp_df["close"] = temp_df["close"] * temp_df["hfq_factor"]
        temp_df["low"] = temp_df["low"] * temp_df["hfq_factor"]
        temp_df = temp_df.iloc[:, :-1]
        temp_df.reset_index(inplace=True)
        temp_df['date'] = pd.to_datetime(temp_df['date']).dt.date
        return temp_df

    if adjust == "qfq":
        res = requests.get(zh_sina_kcb_stock_qfq_url.format(symbol))
        qfq_factor_df = pd.DataFrame(
            eval(res.text.split("=")[1].split("\n")[0])["data"]
        )
        qfq_factor_df.columns = ["date", "qfq_factor"]
        qfq_factor_df.index = pd.to_datetime(qfq_factor_df.date)
        del qfq_factor_df["date"]

        temp_df = pd.merge(
            temp_df, qfq_factor_df, left_index=True, right_index=True, how="left"
        )
        try:
            # try for pandas >= 2.1.0
            temp_df.ffill(inplace=True)
        except Exception as e:
            try:
            # try for pandas < 2.1.0          
                temp_df.fillna(method="ffill", inplace=True)
            except Exception as e:
                print("Error:", e)
        temp_df = temp_df.astype(float)
        temp_df["open"] = temp_df["open"] / temp_df["qfq_factor"]
        temp_df["high"] = temp_df["high"] / temp_df["qfq_factor"]
        temp_df["close"] = temp_df["close"] / temp_df["qfq_factor"]
        temp_df["low"] = temp_df["low"] / temp_df["qfq_factor"]
        temp_df = temp_df.iloc[:, :-1]
        temp_df.reset_index(inplace=True)
        temp_df['date'] = pd.to_datetime(temp_df['date']).dt.date
        return temp_df

    if adjust == "hfq-factor":
        res = requests.get(zh_sina_kcb_stock_hfq_url.format(symbol))
        hfq_factor_df = pd.DataFrame(
            eval(res.text.split("=")[1].split("\n")[0])["data"]
        )
        hfq_factor_df.columns = ["date", "hfq_factor"]
        hfq_factor_df.index = pd.to_datetime(hfq_factor_df.date)
        del hfq_factor_df["date"]
        hfq_factor_df.reset_index(inplace=True)
        hfq_factor_df['date'] = pd.to_datetime(hfq_factor_df['date']).dt.date
        return hfq_factor_df

    if adjust == "qfq-factor":
        res = requests.get(zh_sina_kcb_stock_qfq_url.format(symbol))
        qfq_factor_df = pd.DataFrame(
            eval(res.text.split("=")[1].split("\n")[0])["data"]
        )
        qfq_factor_df.columns = ["date", "qfq_factor"]
        qfq_factor_df.index = pd.to_datetime(qfq_factor_df.date)
        del qfq_factor_df["date"]
        qfq_factor_df.reset_index(inplace=True)
        qfq_factor_df['date'] = pd.to_datetime(qfq_factor_df['date']).dt.date
        return qfq_factor_df


if __name__ == "__main__":
    stock_zh_kcb_daily_qfq_df = stock_zh_kcb_daily(symbol="sh688399", adjust="qfq")
    print(stock_zh_kcb_daily_qfq_df)

    stock_zh_kcb_daily_hfq_df = stock_zh_kcb_daily(symbol="sh688399", adjust="hfq")
    print(stock_zh_kcb_daily_hfq_df)

    stock_zh_kcb_daily_df = stock_zh_kcb_daily(symbol="sh688399", adjust="qfq-factor")
    print(stock_zh_kcb_daily_df)

    stock_zh_kcb_spot_df = stock_zh_kcb_spot()
    print(stock_zh_kcb_spot_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/29 17:00
Desc: 查询期货合约当前时刻的详情
https://finance.sina.com.cn/futures/quotes/V2101.shtml
"""
from io import StringIO

import pandas as pd
import requests


def futures_contract_detail(symbol: str = 'AP2101') -> pd.DataFrame:
    """
    查询期货合约详情
    https://finance.sina.com.cn/futures/quotes/V2101.shtml
    :param symbol: 合约
    :type symbol: str
    :return: 期货合约详情
    :rtype: pandas.DataFrame
    """
    url = f"https://finance.sina.com.cn/futures/quotes/{symbol}.shtml"
    r = requests.get(url)
    r.encoding = 'gb2312'
    temp_df = pd.read_html(StringIO(r.text))[6]
    data_one = temp_df.iloc[:, :2]
    data_one.columns = ['item', 'value']
    data_two = temp_df.iloc[:, 2:4]
    data_two.columns = ['item', 'value']
    data_three = temp_df.iloc[:, 4:]
    data_three.columns = ['item', 'value']
    temp_df = pd.concat(objs=[data_one, data_two, data_three], axis=0, ignore_index=True)
    return temp_df


if __name__ == '__main__':
    futures_contract_detail_df = futures_contract_detail(symbol='IM2402')
    print(futures_contract_detail_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/20 20:51
Desc: 修大成主页-Risk Lab-Realized Volatility; Oxford-Man Institute of Quantitative Finance Realized Library
"""
import json

import pandas as pd
import requests
import urllib3
from bs4 import BeautifulSoup

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def article_oman_rv(symbol: str = "FTSE", index: str = "rk_th2") -> pd.DataFrame:
    """
    Oxford-Man Institute of Quantitative Finance Realized Library 的数据
    :param symbol: str ['AEX', 'AORD', 'BFX', 'BSESN', 'BVLG', 'BVSP', 'DJI', 'FCHI', 'FTMIB', 'FTSE', 'GDAXI', 'GSPTSE', 'HSI', 'IBEX', 'IXIC', 'KS11', 'KSE', 'MXX', 'N225', 'NSEI', 'OMXC20', 'OMXHPI', 'OMXSPI', 'OSEAX', 'RUT', 'SMSI', 'SPX', 'SSEC', 'SSMI', 'STI', 'STOXX50E']
    :param index: str 指标 ['medrv', 'rk_twoscale', 'bv', 'rv10', 'rv5', 'rk_th2', 'rv10_ss', 'rsv', 'rv5_ss', 'bv_ss', 'rk_parzen', 'rsv_ss']
    :return: pandas.DataFrame

    The Oxford-Man Institute's "realised library" contains daily non-parametric measures of how volatility financial assets or indexes were in the past. Each day's volatility measure depends solely on financial data from that day. They are driven by the use of the latest innovations in econometric modelling and theory to design them, while we draw our high frequency data from the Thomson Reuters DataScope Tick History database. Realised measures are not volatility forecasts. However, some researchers use these measures as an input into forecasting models. The aim of this line of research is to make financial markets more transparent by exposing how volatility changes through time.

    This Library is used as the basis of some of our own research, which effects its scope, and is made available here to encourage the more widespread exploitation of these methods. It is given 'as is' and solely for informational purposes, please read the disclaimer.

    The volatility data can be visually explored. We make the complete up-to-date dataset available for download. Lists of assets covered and realized measures available are also available.
    | Symbol    | Name                                      | Earliest Available | Latest Available  |
    |-----------|-------------------------------------------|--------------------|-------------------|
    | .AEX      | AEX index                                 | January 03, 2000   | November 28, 2019 |
    | .AORD     | All Ordinaries                            | January 04, 2000   | November 28, 2019 |
    | .BFX      | Bell 20 Index                             | January 03, 2000   | November 28, 2019 |
    | .BSESN    | S&P BSE Sensex                            | January 03, 2000   | November 28, 2019 |
    | .BVLG     | PSI All-Share Index                       | October 15, 2012   | November 28, 2019 |
    | .BVSP     | BVSP BOVESPA Index                        | January 03, 2000   | November 28, 2019 |
    | .DJI      | Dow Jones Industrial Average              | January 03, 2000   | November 27, 2019 |
    | .FCHI     | CAC 40                                    | January 03, 2000   | November 28, 2019 |
    | .FTMIB    | FTSE MIB                                  | June 01, 2009      | November 28, 2019 |
    | .FTSE     | FTSE 100                                  | January 04, 2000   | November 28, 2019 |
    | .GDAXI    | DAX                                       | January 03, 2000   | November 28, 2019 |
    | .GSPTSE   | S&P/TSX Composite index                   | May 02, 2002       | November 28, 2019 |
    | .HSI      | HANG SENG Index                           | January 03, 2000   | November 28, 2019 |
    | .IBEX     | IBEX 35 Index                             | January 03, 2000   | November 28, 2019 |
    | .IXIC     | Nasdaq 100                                | January 03, 2000   | November 27, 2019 |
    | .KS11     | Korea Composite Stock Price Index (KOSPI) | January 04, 2000   | November 28, 2019 |
    | .KSE      | Karachi SE 100 Index                      | January 03, 2000   | November 28, 2019 |
    | .MXX      | IPC Mexico                                | January 03, 2000   | November 28, 2019 |
    | .N225     | Nikkei 225                                | February 02, 2000  | November 28, 2019 |
    | .NSEI     | NIFTY 50                                  | January 03, 2000   | November 28, 2019 |
    | .OMXC20   | OMX Copenhagen 20 Index                   | October 03, 2005   | November 28, 2019 |
    | .OMXHPI   | OMX Helsinki All Share Index              | October 03, 2005   | November 28, 2019 |
    | .OMXSPI   | OMX Stockholm All Share Index             | October 03, 2005   | November 28, 2019 |
    | .OSEAX    | Oslo Exchange All-share Index             | September 03, 2001 | November 28, 2019 |
    | .RUT      | Russel 2000                               | January 03, 2000   | November 27, 2019 |
    | .SMSI     | Madrid General Index                      | July 04, 2005      | November 28, 2019 |
    | .SPX      | S&P 500 Index                             | January 03, 2000   | November 27, 2019 |
    | .SSEC     | Shanghai Composite Index                  | January 04, 2000   | November 28, 2019 |
    | .SSMI     | Swiss Stock Market Index                  | January 04, 2000   | November 28, 2019 |
    | .STI      | Straits Times Index                       | January 03, 2000   | November 28, 2019 |
    | .STOXX50E | EURO STOXX 50                             | January 03, 2000   | November 28, 2019 |
    """
    url = "https://realized.oxford-man.ox.ac.uk/theme/js/visualization-data.js?20191111113154"
    res = requests.get(url)
    soup = BeautifulSoup(res.text, "lxml")
    soup_text = soup.find("p").get_text()
    data_json = json.loads(soup_text[soup_text.find("{"): soup_text.rfind("};") + 1])
    date_list = data_json[f".{symbol}"]["dates"]
    title_fore = data_json[f".{symbol}"][index]["name"]
    title_last = data_json[f".{symbol}"][index]["measure"]
    title_list = title_fore + "-" + title_last
    temp_df = pd.DataFrame([date_list, data_json[f".{symbol}"][index]["data"]]).T
    temp_df.index = pd.to_datetime(temp_df.iloc[:, 0], unit="ms")
    temp_df = temp_df.iloc[:, 1]
    temp_df.index.name = "date"
    temp_df.name = f"{symbol}-{index}"
    return temp_df


def article_oman_rv_short(symbol: str = "FTSE") -> pd.DataFrame:
    """
    Oxford-Man Institute of Quantitative Finance Realized Library 的数据
    :param symbol: str FTSE: FTSE 100, GDAXI: DAX, RUT: Russel 2000, SPX: S&P 500 Index, STOXX50E: EURO STOXX 50, SSEC: Shanghai Composite Index, N225: Nikkei 225
    :return: pandas.DataFrame

    The Oxford-Man Institute's "realised library" contains daily non-parametric measures of how volatility financial assets or indexes were in the past. Each day's volatility measure depends solely on financial data from that day. They are driven by the use of the latest innovations in econometric modelling and theory to design them, while we draw our high frequency data from the Thomson Reuters DataScope Tick History database. Realised measures are not volatility forecasts. However, some researchers use these measures as an input into forecasting models. The aim of this line of research is to make financial markets more transparent by exposing how volatility changes through time.

    This Library is used as the basis of some of our own research, which effects its scope, and is made available here to encourage the more widespread exploitation of these methods. It is given 'as is' and solely for informational purposes, please read the disclaimer.

    The volatility data can be visually explored. We make the complete up-to-date dataset available for download. Lists of assets covered and realized measures available are also available.
    """
    url = "https://realized.oxford-man.ox.ac.uk/theme/js/front-page-chart.js"
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "realized.oxford-man.ox.ac.uk",
        "Pragma": "no-cache",
        "Referer": "https://realized.oxford-man.ox.ac.uk/?from=groupmessage&isappinstalled=0",
        "Sec-Fetch-Mode": "no-cors",
        "Sec-Fetch-Site": "same-origin",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.97 Safari/537.36",
    }

    res = requests.get(url, headers=headers, verify=False)
    soup = BeautifulSoup(res.text, "lxml")
    soup_text = soup.find("p").get_text()
    data_json = json.loads(soup_text[soup_text.find("{"): soup_text.rfind("}") + 1])
    title_fore = data_json[f".{symbol}"]["name"]
    title_last = data_json[f".{symbol}"]["measure"]
    title_list = title_fore + "-" + title_last
    temp_df = pd.DataFrame(data_json[f".{symbol}"]["data"])
    temp_df.index = pd.to_datetime(temp_df.iloc[:, 0], unit="ms")
    temp_df = temp_df.iloc[:, 1]
    temp_df.index.name = "date"
    temp_df.name = f"{symbol}"
    return temp_df


def article_rlab_rv(symbol: str = "39693") -> pd.DataFrame:
    """
    修大成主页-Risk Lab-Realized Volatility
    :param symbol: str 股票代码
    :return: pandas.DataFrame
    1996-01-02    0.000000
    1996-01-04    0.000000
    1996-01-05    0.000000
    1996-01-09    0.000000
    1996-01-10    0.000000
                    ...
    2019-11-04    0.175107
    2019-11-05    0.185112
    2019-11-06    0.210373
    2019-11-07    0.240808
    2019-11-08    0.199549
    Name: RV, Length: 5810, dtype: float64

    Website
    https://dachxiu.chicagobooth.edu/

    Objective
    We provide up-to-date daily annualized realized volatilities for individual stocks, ETFs, and future contracts, which are estimated from high-frequency data. We are in the process of incorporating equities from global markets.

    Data
    We collect trades at their highest frequencies available (up to every millisecond for US equities after 2007), and clean them using the prevalent national best bid and offer (NBBO) that are available up to every second. The mid-quotes are calculated based on the NBBOs, so their highest sampling frequencies are also up to every second.

    Methodology
    We provide quasi-maximum likelihood estimates of volatility (QMLE) based on moving-average models MA(q), using non-zero returns of transaction prices (or mid-quotes if available) sampled up to their highest frequency available, for days with at least 12 observations. We select the best model (q) using Akaike Information Criterion (AIC). For comparison, we report realized volatility (RV) estimates using 5-minute and 15-minute subsampled returns.

    References
    1. “When Moving-Average Models Meet High-Frequency Data: Uniform Inference on Volatility”, by Rui Da and Dacheng Xiu. 2017.
    2. “Quasi-Maximum Likelihood Estimation of Volatility with High Frequency Data”, by Dacheng Xiu. Journal of Econometrics, 159 (2010), 235-250.
    3. “How Often to Sample A Continuous-time Process in the Presence of Market Microstructure Noise”, by Yacine Aït-Sahalia, Per Mykland, and Lan Zhang. Review of Financial Studies, 18 (2005), 351–416.
    4. “The Distribution of Exchange Rate Volatility”, by Torben Andersen, Tim Bollerslev, Francis X. Diebold, and Paul Labys. Journal of the American Statistical Association, 96 (2001), 42-55.
    5. “Econometric Analysis of Realized Volatility and Its Use in Estimating Stochastic Volatility Models”, by Ole E Barndorff‐Nielsen and Neil Shephard. Journal of the Royal Statistical Society: Series B, 64 (2002), 253-280.
    """
    print("由于服务器在国外, 请稍后, 如果访问失败, 请使用代理工具")
    url = "https://dachxiu.chicagobooth.edu/data.php"
    payload = {"ticker": symbol}
    res = requests.get(url, params=payload, verify=False)
    soup = BeautifulSoup(res.text, "lxml")
    title_fore = (
        pd.DataFrame(soup.find("p").get_text().split(symbol)).iloc[0, 0].strip()
    )
    title_list = (
        pd.DataFrame(soup.find("p").get_text().split(symbol))
        .iloc[1, 0]
        .strip()
        .split("\n")
    )
    title_list.insert(0, title_fore)
    temp_df = pd.DataFrame(soup.find("p").get_text().split(symbol)).iloc[2:, :]
    temp_df = temp_df.iloc[:, 0].str.split(" ", expand=True)
    temp_df = temp_df.iloc[:, 1:]
    temp_df.iloc[:, -1] = temp_df.iloc[:, -1].str.replace(r"\n", "")
    temp_df.reset_index(inplace=True)
    temp_df.index = pd.to_datetime(temp_df.iloc[:, 1], format="%Y%m%d", errors="coerce")
    temp_df = temp_df.iloc[:, 1:]
    data_se = temp_df.iloc[:, 1]
    data_se.name = "RV"
    temp_df = data_se.astype("float", errors="ignore")
    temp_df.index.name = "date"
    return temp_df


if __name__ == "__main__":
    article_rlab_rv_df = article_rlab_rv(symbol="39693")
    print(article_rlab_rv_df)

    article_oman_rv_short_df = article_oman_rv_short(symbol="FTSE")
    print(article_oman_rv_short_df)

    article_oman_rv_df = article_oman_rv(symbol="FTSE", index="rk_th2")
    print(article_oman_rv_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/9/15 15:30
Desc: 东方财富网-数据中心-特色数据-商誉
东方财富网-数据中心-特色数据-商誉-A股商誉市场概况: https://data.eastmoney.com/sy/scgk.html
东方财富网-数据中心-特色数据-商誉-商誉减值预期明细: https://data.eastmoney.com/sy/yqlist.html
东方财富网-数据中心-特色数据-商誉-个股商誉减值明细: https://data.eastmoney.com/sy/jzlist.html
东方财富网-数据中心-特色数据-商誉-个股商誉明细: https://data.eastmoney.com/sy/list.html
东方财富网-数据中心-特色数据-商誉-行业商誉: https://data.eastmoney.com/sy/hylist.html
"""

import pandas as pd
import requests

from byteapp_ak.utils.tqdm import get_tqdm


def stock_sy_profile_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-商誉-A股商誉市场概况
    https://data.eastmoney.com/sy/scgk.html
    :return: A股商誉市场概况
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_GOODWILL_MARKETSTATISTICS",
        "token": "894050c76af8597a853f5b408b759f5d",
        "columns": "ALL",
        "filter": """((GOODWILL_STATE="1")( | IMPAIRMENT_STATE="1"))(TRADE_BOARD="all")""",
    }

    r = requests.get(url, params=params)
    data_json = r.json()
    data_df = pd.DataFrame(data_json["result"]["data"])
    data_df.columns = [
        "_",
        "报告期",
        "商誉",
        "商誉减值",
        "净资产",
        "商誉占净资产比例",
        "商誉减值占净资产比例",
        "净利润规模",
        "商誉减值占净利润比例",
        "_",
        "_",
    ]
    data_df = data_df[
        [
            "报告期",
            "商誉",
            "商誉减值",
            "净资产",
            "商誉占净资产比例",
            "商誉减值占净资产比例",
            "净利润规模",
            "商誉减值占净利润比例",
        ]
    ]
    data_df["报告期"] = pd.to_datetime(data_df["报告期"], errors="coerce").dt.date
    data_df["商誉"] = pd.to_numeric(data_df["商誉"], errors="coerce")
    data_df["商誉减值"] = pd.to_numeric(data_df["商誉减值"], errors="coerce")
    data_df["净资产"] = pd.to_numeric(data_df["净资产"], errors="coerce")
    data_df["商誉占净资产比例"] = pd.to_numeric(
        data_df["商誉占净资产比例"], errors="coerce"
    )
    data_df["商誉减值占净资产比例"] = pd.to_numeric(
        data_df["商誉减值占净资产比例"], errors="coerce"
    )
    data_df["净利润规模"] = pd.to_numeric(data_df["净利润规模"], errors="coerce")
    data_df["商誉减值占净利润比例"] = pd.to_numeric(
        data_df["商誉减值占净利润比例"], errors="coerce"
    )
    data_df.sort_values(["报告期"], inplace=True, ignore_index=True)
    return data_df


def stock_sy_yq_em(date: str = "20240630") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-商誉-商誉减值预期明细
    https://data.eastmoney.com/sy/yqlist.html
    :param date: 参考网站指定的数据日期
    :type date: str
    :return: 商誉减值预期明细
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "NOTICE_DATE,SECURITY_CODE",
        "sortTypes": "-1,-1",
        "pageSize": "5000",
        "pageNumber": "1",
        "columns": "ALL",
        "token": "894050c76af8597a853f5b408b759f5d",
        "reportName": "RPT_GOODWILL_STOCKPREDICT",
        "filter": f"""(REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    total_page = int(data_json["result"]["pages"])
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df["index"] + 1
    big_df.rename(
        columns={
            "index": "序号",
            "SECUCODE": "-",
            "SECURITY_CODE": "股票代码",
            "ORG_CODE": "-",
            "SECURITY_NAME_ABBR": "股票简称",
            "TRADE_MARKET": "交易市场",
            "INDUSTRY_NAME": "-",
            "INDUSTRY_CODE": "-",
            "NOTICE_DATE": "公告日期",
            "REPORT_DATE": "数据日期",
            "PE_REPORT_DATE": "-",
            "PREDICT_NETPROFIT_LOWER": "预计净利润-下限",
            "PREDICT_NETPROFIT_UPPER": "预计净利润-上限",
            "PERFORM_CHANGE_UPPER": "业绩变动幅度-上限",
            "PERFORM_CHANGE_LOWER": "业绩变动幅度-下限",
            "PERFORM_CHANGE": "-",
            "PERFORM_CHANGE_EXPLAIN": "业绩变动原因",
            "PREDICT_TYPE": "-",
            "PREDICT_INDICATOR_CODE": "-",
            "PREDICT_PERIOD": "-",
            "PE_SAMEREPORT_NETPROFIT": "上年度同期净利润",
            "NETPROFIT": "-",
            "PE_GOODWILL": "上年商誉",
            "NEWEST_REPORT_DATE": "最新商誉报告期",
            "NEWEST_GOODWILL": "最新一期商誉",
            "PE_SAMEREPORT_DATE": "-",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "业绩变动原因",
            "最新商誉报告期",
            "最新一期商誉",
            "上年商誉",
            "预计净利润-下限",
            "预计净利润-上限",
            "业绩变动幅度-下限",
            "业绩变动幅度-上限",
            "上年度同期净利润",
            "公告日期",
            "交易市场",
        ]
    ]
    big_df["交易市场"] = big_df["交易市场"].map(
        {"shzb": "沪市主板", "kcb": "科创板", "szzb": "深市主板", "cyb": "创业板"}
    )
    big_df["最新商誉报告期"] = pd.to_datetime(
        big_df["最新商誉报告期"], errors="coerce"
    ).dt.date
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    big_df["最新一期商誉"] = pd.to_numeric(big_df["最新一期商誉"], errors="coerce")
    big_df["上年商誉"] = pd.to_numeric(big_df["上年商誉"], errors="coerce")
    big_df["预计净利润-下限"] = pd.to_numeric(
        big_df["预计净利润-下限"], errors="coerce"
    )
    big_df["预计净利润-上限"] = pd.to_numeric(
        big_df["预计净利润-上限"], errors="coerce"
    )
    big_df["业绩变动幅度-下限"] = pd.to_numeric(
        big_df["业绩变动幅度-下限"], errors="coerce"
    )
    big_df["业绩变动幅度-上限"] = pd.to_numeric(
        big_df["业绩变动幅度-上限"], errors="coerce"
    )
    big_df["上年度同期净利润"] = pd.to_numeric(
        big_df["上年度同期净利润"], errors="coerce"
    )
    return big_df


def stock_sy_jz_em(date: str = "20240630") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-商誉-个股商誉减值明细
    https://data.eastmoney.com/sy/jzlist.html
    :param date: 参考网站指定的数据日期
    :type date: str
    :return: 个股商誉减值明细
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "GOODWILL_CHANGE",
        "sortTypes": "-1",
        "pageSize": "5000",
        "pageNumber": "1",
        "columns": "ALL",
        "token": "894050c76af8597a853f5b408b759f5d",
        "reportName": "RPT_GOODWILL_STOCKDETAILS",
        "filter": f"""(REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    total_page = int(data_json["result"]["pages"])
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df["index"] + 1
    big_df.rename(
        columns={
            "index": "序号",
            "SECUCODE": "-",
            "SECURITY_CODE": "股票代码",
            "SECURITY_NAME_ABBR": "股票简称",
            "ORG_CODE": "-",
            "LISTING_DATE": "-",
            "LISTING_STATE": "-",
            "TRADE_BOARD": "交易市场",
            "GOODWILL": "商誉",
            "DATE_TYPE": "-",
            "REPORT_TYPE_CODE": "-",
            "DATA_ADJUST_TYPE": "-",
            "NOTICE_DATE": "公告日期",
            "REPORT_DATE": "数据日期",
            "GOODWILL_PRE": "-",
            "GOODWILL_CHANGE": "商誉减值",
            "SUMSHEQUITY": "-",
            "SUMSHEQUITY_RATIO": "商誉占净资产比例",
            "SE_CHANGE_RATIO": "商誉减值占净资产比例",
            "PARENTNETPROFIT": "净利润",
            "PNP_CHANGE_RATIO": "商誉减值占净利润比例",
            "PNP_YOY_RATIO": "-",
            "INDUSTRY_CFT": "-",
            "INDUSTRY_CFTCODE": "-",
            "IS_SHOW": "-",
            "MAXREPORTDATE": "-",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "商誉",
            "商誉减值",
            "商誉占净资产比例",
            "商誉减值占净资产比例",
            "净利润",
            "商誉减值占净利润比例",
            "公告日期",
            "交易市场",
        ]
    ]
    big_df["交易市场"] = big_df["交易市场"].map(
        {"shzb": "沪市主板", "kcb": "科创板", "szzb": "深市主板", "cyb": "创业板"}
    )
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    big_df["商誉"] = pd.to_numeric(big_df["商誉"], errors="coerce")
    big_df["商誉减值"] = pd.to_numeric(big_df["商誉减值"], errors="coerce")
    big_df["商誉占净资产比例"] = pd.to_numeric(
        big_df["商誉占净资产比例"], errors="coerce"
    )
    big_df["商誉减值占净资产比例"] = pd.to_numeric(
        big_df["商誉减值占净资产比例"], errors="coerce"
    )
    big_df["净利润"] = pd.to_numeric(big_df["净利润"], errors="coerce")
    big_df["商誉减值占净利润比例"] = pd.to_numeric(
        big_df["商誉减值占净利润比例"], errors="coerce"
    )
    big_df["商誉减值占净利润比例"] = pd.to_numeric(
        big_df["商誉减值占净利润比例"], errors="coerce"
    )
    return big_df


def stock_sy_em(date: str = "20231231") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-商誉-个股商誉明细
    https://data.eastmoney.com/sy/list.html
    :param date: 参考网站指定的数据日期
    :type date: str
    :return: 个股商誉明细
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "NOTICE_DATE,SECURITY_CODE",
        "sortTypes": "-1,-1",
        "pageSize": "5000",
        "pageNumber": "1",
        "columns": "ALL",
        "token": "894050c76af8597a853f5b408b759f5d",
        "reportName": "RPT_GOODWILL_STOCKDETAILS",
        "filter": f"""(REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    total_page = int(data_json["result"]["pages"])
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df["index"] + 1
    big_df.rename(
        columns={
            "index": "序号",
            "SECUCODE": "-",
            "SECURITY_CODE": "股票代码",
            "SECURITY_NAME_ABBR": "股票简称",
            "ORG_CODE": "-",
            "LISTING_DATE": "-",
            "LISTING_STATE": "-",
            "TRADE_BOARD": "交易市场",
            "GOODWILL": "商誉",
            "DATE_TYPE": "-",
            "REPORT_TYPE_CODE": "-",
            "DATA_ADJUST_TYPE": "-",
            "NOTICE_DATE": "公告日期",
            "REPORT_DATE": "数据日期",
            "GOODWILL_PRE": "上年商誉",
            "GOODWILL_CHANGE": "商誉减值",
            "SUMSHEQUITY": "-",
            "SUMSHEQUITY_RATIO": "商誉占净资产比例",
            "SE_CHANGE_RATIO": "商誉减值占净资产比例",
            "PARENTNETPROFIT": "净利润",
            "PNP_CHANGE_RATIO": "商誉减值占净利润比例",
            "PNP_YOY_RATIO": "净利润同比",
            "INDUSTRY_CFT": "-",
            "INDUSTRY_CFTCODE": "-",
            "IS_SHOW": "-",
            "MAXREPORTDATE": "-",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "商誉",
            "商誉占净资产比例",
            "净利润",
            "净利润同比",
            "上年商誉",
            "公告日期",
            "交易市场",
        ]
    ]
    big_df["交易市场"] = big_df["交易市场"].map(
        {"shzb": "沪市主板", "kcb": "科创板", "szzb": "深市主板", "cyb": "创业板"}
    )
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    big_df["商誉"] = pd.to_numeric(big_df["商誉"], errors="coerce")
    big_df["商誉占净资产比例"] = pd.to_numeric(
        big_df["商誉占净资产比例"], errors="coerce"
    )
    big_df["净利润"] = pd.to_numeric(big_df["净利润"], errors="coerce")
    big_df["净利润同比"] = pd.to_numeric(big_df["净利润同比"], errors="coerce")
    big_df["上年商誉"] = pd.to_numeric(big_df["上年商誉"], errors="coerce")
    return big_df


def stock_sy_hy_em(date: str = "20231231") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-商誉-行业商誉
    https://data.eastmoney.com/sy/hylist.html
    :param date: 参考网站指定的数据日期
    :type date: str
    :return: 个股商誉明细
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "SUMSHEQUITY_RATIO",
        "sortTypes": "-1",
        "pageSize": "5000",
        "pageNumber": "1",
        "columns": "ALL",
        "token": "894050c76af8597a853f5b408b759f5d",
        "reportName": "RPT_GOODWILL_INDUSTATISTICS",
        "filter": f"""(REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    total_page = int(data_json["result"]["pages"])
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True, drop=True)
    big_df.rename(
        columns={
            "REPORT_DATE": "数据日期",
            "INDUSTRY_NAME": "行业名称",
            "INDUSTRY_CODE": "-",
            "ORG_NUM": "公司家数",
            "GOODWILL": "商誉规模",
            "GOODWILL_CHANGE": "-",
            "SUMSHEQUITY": "净资产",
            "SUMSHEQUITY_RATIO": "商誉规模占净资产规模比例",
            "SE_CHANGE_RATIO": "-",
            "PARENTNETPROFIT": "净利润规模",
            "PNP_CHANGE_RATIO": "-",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "行业名称",
            "公司家数",
            "商誉规模",
            "净资产",
            "商誉规模占净资产规模比例",
            "净利润规模",
        ]
    ]
    big_df["公司家数"] = pd.to_numeric(big_df["公司家数"], errors="coerce")
    big_df["商誉规模"] = pd.to_numeric(big_df["商誉规模"], errors="coerce")
    big_df["净资产"] = pd.to_numeric(big_df["净资产"], errors="coerce")
    big_df["商誉规模占净资产规模比例"] = pd.to_numeric(
        big_df["商誉规模占净资产规模比例"], errors="coerce"
    )
    big_df["净利润规模"] = pd.to_numeric(big_df["净利润规模"], errors="coerce")
    return big_df


if __name__ == "__main__":
    stock_em_sy_profile_df = stock_sy_profile_em()
    print(stock_em_sy_profile_df)

    stock_sy_yq_em_df = stock_sy_yq_em(date="20240630")
    print(stock_sy_yq_em_df)

    stock_sy_jz_em_df = stock_sy_jz_em(date="20240630")
    print(stock_sy_jz_em_df)

    stock_sy_em_df = stock_sy_em(date="20240630")
    print(stock_sy_em_df)

    stock_sy_hy_em_df = stock_sy_hy_em(date="20231231")
    print(stock_sy_hy_em_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2022/4/7 17:36
Desc: http://kcb.sse.com.cn/renewal/xmxq/index.shtml?auditId=926&anchor_type=0
"""
import requests
import pandas as pd

# TODO


def stock_kcb_detail_renewal():
    url = "http://query.sse.com.cn/commonSoaQuery.do"
    params = {
        'isPagination': 'true',
        'sqlId': 'SH_XM_LB',
        'stockAuditNum': '926',  # 每次更新该字段就可以
        '_': '1649324745607',
    }
    headers = {
        'Host': 'query.sse.com.cn',
        'Pragma': 'no-cache',
        'Referer': 'http://kcb.sse.com.cn/',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/100.0.4896.60 Safari/537.36'
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['result'])
    # 处理下 temp_df 里面的字段就可以了
    print(temp_df)




#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/8/18 15:32
Desc: 东方财富网-数据中心-股东大会
https://data.eastmoney.com/gddh/
"""
import pandas as pd
import requests
from tqdm import tqdm


def stock_gddh_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-股东大会
    https://data.eastmoney.com/gddh/
    :return: 股东大会
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "NOTICE_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_GENERALMEETING_DETAIL",
        "columns": "SECURITY_CODE,SECURITY_NAME_ABBR,MEETING_TITLE,START_ADJUST_DATE,EQUITY_RECORD_DATE,ONSITE_RECORD_DATE,DECISION_NOTICE_DATE,NOTICE_DATE,WEB_START_DATE,WEB_END_DATE,SERIAL_NUM,PROPOSAL",
        "filter": '(IS_LASTDATE="1")',
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat([big_df, temp_df], axis=0, ignore_index=True)
    big_df.rename(
        columns={
            "SECURITY_CODE": "代码",
            "SECURITY_NAME_ABBR": "简称",
            "MEETING_TITLE": "股东大会名称",
            "START_ADJUST_DATE": "召开开始日",
            "EQUITY_RECORD_DATE": "股权登记日",
            "ONSITE_RECORD_DATE": "现场登记日",
            "DECISION_NOTICE_DATE": "决议公告日",
            "NOTICE_DATE": "公告日",
            "WEB_START_DATE": "网络投票时间-开始日",
            "WEB_END_DATE": "网络投票时间-结束日",
            "SERIAL_NUM": "序列号",
            "PROPOSAL": "提案",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "代码",
            "简称",
            "股东大会名称",
            "召开开始日",
            "股权登记日",
            "现场登记日",
            "网络投票时间-开始日",
            "网络投票时间-结束日",
            "决议公告日",
            "公告日",
            "序列号",
            "提案",
        ]
    ]
    big_df['召开开始日'] = pd.to_datetime(big_df['召开开始日'], errors="coerce").dt.date
    big_df['股权登记日'] = pd.to_datetime(big_df['股权登记日'], errors="coerce").dt.date
    big_df['现场登记日'] = pd.to_datetime(big_df['现场登记日'], errors="coerce").dt.date
    big_df['网络投票时间-开始日'] = pd.to_datetime(big_df['网络投票时间-开始日'], errors="coerce").dt.date
    big_df['网络投票时间-结束日'] = pd.to_datetime(big_df['网络投票时间-结束日'], errors="coerce").dt.date
    big_df['决议公告日'] = pd.to_datetime(big_df['决议公告日'], errors="coerce").dt.date
    big_df['公告日'] = pd.to_datetime(big_df['公告日'], errors="coerce").dt.date
    big_df["提案"] = big_df["提案"].str.replace("\r\n2", "").str.replace("\r\n3", "")
    return big_df


if __name__ == "__main__":
    stock_gddh_em_df = stock_gddh_em()
    print(stock_gddh_em_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/9/1 16:20
Desc: 新浪财经-基金规模
https://vip.stock.finance.sina.com.cn/fund_center/index.html#jjgmall
"""

import pandas as pd
import requests

from byteapp_ak.utils import demjson


def fund_scale_open_sina(symbol: str = "股票型基金") -> pd.DataFrame:
    """
    新浪财经-基金数据中心-基金规模-开放式基金
    https://vip.stock.finance.sina.com.cn/fund_center/index.html#jjhqetf
    :param symbol: choice of {"股票型基金", "混合型基金", "债券型基金", "货币型基金", "QDII基金"}
    :type symbol: str
    :return: 基金规模
    :rtype: pandas.DataFrame
    """
    fund_map = {
        "股票型基金": "2",
        "混合型基金": "1",
        "债券型基金": "3",
        "货币型基金": "5",
        "QDII基金": "6",
    }
    url = (
        "http://vip.stock.finance.sina.com.cn/fund_center/data/jsonp.php/IO.XSRV2."
        "CallbackList['J2cW8KXheoWKdSHc']/NetValueReturn_Service.NetValueReturnOpen"
    )
    params = {
        "page": "1",
        "num": "10000",
        "sort": "zmjgm",
        "asc": "0",
        "ccode": "",
        "type2": fund_map[symbol],
        "type3": "",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("({") + 1 : -2])
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    temp_df.rename(
        columns={
            "index": "序号",
            "symbol": "基金代码",
            "sname": "基金简称",
            "per_nav": "-",
            "total_nav": "-",
            "three_month": "-",
            "six_month": "-",
            "one_year": "-",
            "form_year": "-",
            "form_start": "-",
            "name": "-",
            "zmjgm": "总募集规模",
            "clrq": "成立日期",
            "jjjl": "基金经理",
            "dwjz": "单位净值",
            "ljjz": "-",
            "jzrq": "更新日期",
            "zjzfe": "最近总份额",
            "jjglr_code": "-",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "单位净值",
            "总募集规模",
            "最近总份额",
            "成立日期",
            "基金经理",
            "更新日期",
        ]
    ]
    temp_df["成立日期"] = pd.to_datetime(temp_df["成立日期"], errors="coerce").dt.date
    temp_df["更新日期"] = pd.to_datetime(temp_df["更新日期"], errors="coerce").dt.date
    temp_df["单位净值"] = pd.to_numeric(temp_df["单位净值"], errors="coerce")
    temp_df["总募集规模"] = pd.to_numeric(temp_df["总募集规模"], errors="coerce")
    temp_df["最近总份额"] = pd.to_numeric(temp_df["最近总份额"], errors="coerce")
    return temp_df


def fund_scale_close_sina() -> pd.DataFrame:
    """
    新浪财经-基金数据中心-基金规模-封闭式基金
    https://vip.stock.finance.sina.com.cn/fund_center/index.html#jjhqetf
    :return: 基金规模
    :rtype: pandas.DataFrame
    """
    url = (
        "http://vip.stock.finance.sina.com.cn/fund_center/data/jsonp.php/IO.XSRV2."
        "CallbackList['_bjN6KvXOkfPy2Bu']/NetValueReturn_Service.NetValueReturnClose"
    )
    params = {
        "page": "1",
        "num": "1000",
        "sort": "zmjgm",
        "asc": "0",
        "ccode": "",
        "type2": "",
        "type3": "",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("({") + 1 : -2])
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    temp_df.rename(
        columns={
            "index": "序号",
            "symbol": "基金代码",
            "sname": "基金简称",
            "per_nav": "-",
            "total_nav": "-",
            "three_month": "-",
            "six_month": "-",
            "one_year": "-",
            "form_year": "-",
            "form_start": "-",
            "name": "-",
            "zmjgm": "总募集规模",
            "clrq": "成立日期",
            "jjjl": "基金经理",
            "dwjz": "单位净值",
            "ljjz": "-",
            "jzrq": "更新日期",
            "zjzfe": "最近总份额",
            "jjglr_code": "-",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "单位净值",
            "总募集规模",
            "最近总份额",
            "成立日期",
            "基金经理",
            "更新日期",
        ]
    ]
    temp_df["成立日期"] = pd.to_datetime(temp_df["成立日期"], errors="coerce").dt.date
    temp_df["更新日期"] = pd.to_datetime(temp_df["更新日期"], errors="coerce").dt.date
    temp_df["单位净值"] = pd.to_numeric(temp_df["单位净值"], errors="coerce")
    temp_df["总募集规模"] = pd.to_numeric(temp_df["总募集规模"], errors="coerce")
    temp_df["最近总份额"] = pd.to_numeric(temp_df["最近总份额"], errors="coerce")
    return temp_df


def fund_scale_structured_sina() -> pd.DataFrame:
    """
    新浪财经-基金数据中心-基金规模-分级子基金
    https://vip.stock.finance.sina.com.cn/fund_center/index.html#jjgmfjall
    :return: 基金规模
    :rtype: pandas.DataFrame
    """
    url = (
        "http://vip.stock.finance.sina.com.cn/fund_center/data/jsonp.php/IO.XSRV2."
        "CallbackList['cRrwseM7NWX68rDa']/NetValueReturn_Service.NetValueReturnCX"
    )
    params = {
        "page": "1",
        "num": "1000",
        "sort": "zmjgm",
        "asc": "0",
        "ccode": "",
        "type2": "",
        "type3": "",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("({") + 1 : -2])
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    temp_df.rename(
        columns={
            "index": "序号",
            "symbol": "基金代码",
            "sname": "基金简称",
            "per_nav": "-",
            "total_nav": "-",
            "three_month": "-",
            "six_month": "-",
            "one_year": "-",
            "form_year": "-",
            "form_start": "-",
            "name": "-",
            "zmjgm": "总募集规模",
            "clrq": "成立日期",
            "jjjl": "基金经理",
            "dwjz": "单位净值",
            "ljjz": "-",
            "jzrq": "更新日期",
            "zjzfe": "最近总份额",
            "jjglr_code": "-",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "单位净值",
            "总募集规模",
            "最近总份额",
            "成立日期",
            "基金经理",
            "更新日期",
        ]
    ]
    temp_df["成立日期"] = pd.to_datetime(temp_df["成立日期"], errors="coerce").dt.date
    temp_df["更新日期"] = pd.to_datetime(temp_df["更新日期"], errors="coerce").dt.date
    temp_df["单位净值"] = pd.to_numeric(temp_df["单位净值"], errors="coerce")
    temp_df["总募集规模"] = pd.to_numeric(temp_df["总募集规模"], errors="coerce")
    temp_df["最近总份额"] = pd.to_numeric(temp_df["最近总份额"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    fund_scale_open_sina_df = fund_scale_open_sina(symbol="股票型基金")
    print(fund_scale_open_sina_df)

    fund_scale_close_sina_df = fund_scale_close_sina()
    print(fund_scale_close_sina_df)

    fund_scale_structured_sina_df = fund_scale_structured_sina()
    print(fund_scale_structured_sina_df)

    fund_scale_open_sina_df = fund_scale_open_sina(symbol="股票型基金")
    print(fund_scale_open_sina_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/1/23 10:21
Desc: 新浪财经-商品期权
https://stock.finance.sina.com.cn/futures/view/optionsDP.php
"""
import pandas as pd
import requests
from bs4 import BeautifulSoup

from byteapp_ak.utils import demjson


def option_commodity_contract_sina(symbol: str = "玉米期权") -> pd.DataFrame:
    """
    当前可以查询的期权品种的合约日期
    https://stock.finance.sina.com.cn/futures/view/optionsDP.php
    :param symbol: choice of {"豆粕期权", "玉米期权", "铁矿石期权", "棉花期权", "白糖期权", "PTA期权", "甲醇期权", "橡胶期权", "沪铜期权", "黄金期权", "菜籽粕期权", "液化石油气期权", "动力煤期权", "菜籽油期权", "花生期权"}
    :type symbol: str
    :return: e.g., {'黄金期权': ['au2012', 'au2008', 'au2010', 'au2104', 'au2102', 'au2106', 'au2108']}
    :rtype: dict
    """
    url = (
        "https://stock.finance.sina.com.cn/futures/view/optionsDP.php/pg_o/dce"
    )
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    url_list = [
        item.find("a")["href"]
        for item in soup.find_all("li", attrs={"class": "active"})
        if item.find("a") is not None
    ]
    commodity_list = [
        item.find("a").text
        for item in soup.find_all("li", attrs={"class": "active"})
        if item.find("a") is not None
    ]
    comm_list_dict = {
        key: value for key, value in zip(commodity_list, url_list)
    }
    url = "https://stock.finance.sina.com.cn" + comm_list_dict[symbol]
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    symbol = (
        soup.find(attrs={"id": "option_symbol"})
        .find(attrs={"class": "selected"})
        .text
    )
    contract = [
        item.text
        for item in soup.find(attrs={"id": "option_suffix"}).find_all("li")
    ]
    temp_df = pd.DataFrame({symbol: contract})
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1
    temp_df.columns = ["序号", "合约"]
    return temp_df


def option_commodity_contract_table_sina(
    symbol: str = "黄金期权", contract: str = "au2204"
) -> pd.DataFrame:
    """
    当前所有期权合约, 包括看涨期权合约和看跌期权合约
    https://stock.finance.sina.com.cn/futures/view/optionsDP.php
    :param symbol: choice of {"豆粕期权", "玉米期权", "铁矿石期权", "棉花期权", "白糖期权", "PTA期权", "甲醇期权", "橡胶期权", "沪铜期权", "黄金期权", "菜籽粕期权", "液化石油气期权", "动力煤期权", "菜籽油期权", "花生期权"}
    :type symbol: str
    :param contract: e.g., 'au2012'
    :type contract: str
    :return: 合约实时行情
    :rtype: pandas.DataFrame
    """
    url = (
        "https://stock.finance.sina.com.cn/futures/view/optionsDP.php/pg_o/dce"
    )
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    url_list = [
        item.find("a")["href"]
        for item in soup.find_all("li", attrs={"class": "active"})
        if item.find("a") is not None
    ]
    commodity_list = [
        item.find("a").text
        for item in soup.find_all("li", attrs={"class": "active"})
        if item.find("a") is not None
    ]
    comm_list_dict = {
        key: value for key, value in zip(commodity_list, url_list)
    }
    url = "https://stock.finance.sina.com.cn/futures/api/openapi.php/OptionService.getOptionData"
    params = {
        "type": "futures",
        "product": comm_list_dict[symbol].split("/")[-2],
        "exchange": comm_list_dict[symbol].split("/")[-1],
        "pinzhong": contract,
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    up_df = pd.DataFrame(data_json["result"]["data"]["up"])
    down_df = pd.DataFrame(data_json["result"]["data"]["down"])
    temp_df = pd.concat([up_df, down_df], axis=1)
    temp_df.columns = [
        "看涨合约-买量",
        "看涨合约-买价",
        "看涨合约-最新价",
        "看涨合约-卖价",
        "看涨合约-卖量",
        "看涨合约-持仓量",
        "看涨合约-涨跌",
        "行权价",
        "看涨合约-看涨期权合约",
        "看跌合约-买量",
        "看跌合约-买价",
        "看跌合约-最新价",
        "看跌合约-卖价",
        "看跌合约-卖量",
        "看跌合约-持仓量",
        "看跌合约-涨跌",
        "看跌合约-看跌期权合约",
    ]
    temp_df["看涨合约-买量"] = pd.to_numeric(temp_df["看涨合约-买量"], errors="coerce")
    temp_df["看涨合约-买价"] = pd.to_numeric(temp_df["看涨合约-买价"], errors="coerce")
    temp_df["看涨合约-最新价"] = pd.to_numeric(temp_df["看涨合约-最新价"], errors="coerce")
    temp_df["看涨合约-卖价"] = pd.to_numeric(temp_df["看涨合约-卖价"], errors="coerce")
    temp_df["看涨合约-卖量"] = pd.to_numeric(temp_df["看涨合约-卖量"], errors="coerce")
    temp_df["看涨合约-持仓量"] = pd.to_numeric(temp_df["看涨合约-持仓量"], errors="coerce")
    temp_df["看涨合约-涨跌"] = pd.to_numeric(temp_df["看涨合约-涨跌"], errors="coerce")
    temp_df["行权价"] = pd.to_numeric(temp_df["行权价"], errors="coerce")
    temp_df["看跌合约-买量"] = pd.to_numeric(temp_df["看跌合约-买量"], errors="coerce")
    temp_df["看跌合约-买价"] = pd.to_numeric(temp_df["看跌合约-买价"], errors="coerce")
    temp_df["看跌合约-最新价"] = pd.to_numeric(temp_df["看跌合约-最新价"], errors="coerce")
    temp_df["看跌合约-卖价"] = pd.to_numeric(temp_df["看跌合约-卖价"], errors="coerce")
    temp_df["看跌合约-卖量"] = pd.to_numeric(temp_df["看跌合约-卖量"], errors="coerce")
    temp_df["看跌合约-持仓量"] = pd.to_numeric(temp_df["看跌合约-持仓量"], errors="coerce")
    temp_df["看跌合约-涨跌"] = pd.to_numeric(temp_df["看跌合约-涨跌"], errors="coerce")
    return temp_df


def option_commodity_hist_sina(symbol: str = "au2012C392") -> pd.DataFrame:
    """
    合约历史行情-日频
    https://stock.finance.sina.com.cn/futures/view/optionsDP.php
    :param symbol: return of option_sina_option_commodity_contract_list(symbol="黄金期权", contract="au2012"), 看涨期权合约 filed
    :type symbol: str
    :return: 合约历史行情-日频
    :rtype: pandas.DataFrame
    """
    url = "https://stock.finance.sina.com.cn/futures/api/jsonp.php/var%20_m2009C30002020_7_17=/FutureOptionAllService.getOptionDayline"
    params = {"symbol": symbol}
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("[") : -2])
    temp_df = pd.DataFrame(data_json)
    temp_df.columns = ["open", "high", "low", "close", "volume", "date"]
    temp_df = temp_df[["date", "open", "high", "low", "close", "volume"]]
    temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
    temp_df["open"] = pd.to_numeric(temp_df["open"])
    temp_df["high"] = pd.to_numeric(temp_df["high"])
    temp_df["low"] = pd.to_numeric(temp_df["low"])
    temp_df["close"] = pd.to_numeric(temp_df["close"])
    temp_df["volume"] = pd.to_numeric(temp_df["volume"])
    return temp_df


if __name__ == "__main__":
    option_commodity_contract_sina_df = option_commodity_contract_sina(
        symbol="棉花期权"
    )
    print(option_commodity_contract_sina_df)

    option_commodity_contract_table_sina_df = (
        option_commodity_contract_table_sina(symbol="棉花期权", contract="cf2301")
    )
    print(option_commodity_contract_table_sina_df)

    option_commodity_hist_sina_df = option_commodity_hist_sina(
        symbol="cf2301P21600"
    )
    print(option_commodity_hist_sina_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/12/5 19:00
Desc: 国泰君安期货-交易日历数据表
https://www.gtjaqh.com/pc/calendar.html
"""
from io import StringIO

import pandas as pd
import requests


def futures_rule(date: str = "20231205") -> pd.DataFrame:
    """
    国泰君安期货-交易日历数据表
    https://www.gtjaqh.com/pc/calendar.html
    :param date: 需要指定为交易日, 且是近期的日期
    :type date: str
    :return: 交易日历数据
    :rtype: pandas.DataFrame
    """
    import urllib3
    urllib3.disable_warnings()
    url = " https://www.gtjaqh.com/pc/calendar"
    params = {"date": f"{date}"}
    r = requests.get(url, params=params, verify=False)
    big_df = pd.read_html(StringIO(r.text), header=1)[0]
    big_df["交易保证金比例"] = big_df["交易保证金比例"].str.strip("%")
    big_df["交易保证金比例"] = pd.to_numeric(big_df["交易保证金比例"], errors="coerce")
    big_df["涨跌停板幅度"] = big_df["涨跌停板幅度"].str.strip("%")
    big_df["涨跌停板幅度"] = pd.to_numeric(big_df["涨跌停板幅度"], errors="coerce")
    big_df["合约乘数"] = pd.to_numeric(big_df["合约乘数"], errors="coerce")
    big_df["最小变动价位"] = pd.to_numeric(big_df["最小变动价位"], errors="coerce")
    big_df["限价单每笔最大下单手数"] = pd.to_numeric(big_df["限价单每笔最大下单手数"], errors="coerce")
    return big_df


if __name__ == "__main__":
    futures_rule_df = futures_rule(date="20231205")
    print(futures_rule_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/2/28 16:00
Desc: 广州期货交易所-业务/服务-合约信息
http://www.gfex.com.cn/gfex/hyxx/ywcs.shtml
"""
import pandas as pd
import requests


def futures_contract_info_gfex() -> pd.DataFrame:
    """
    广州期货交易所-业务/服务-合约信息
    http://www.gfex.com.cn/gfex/hyxx/ywcs.shtml
    :return: 交易参数汇总查询
    :rtype: pandas.DataFrame
    """
    url = f"http://www.gfex.com.cn/u/interfacesWebTtQueryContractInfo/loadList"
    params = {
        'variety': '',
        'trade_type': '0',
    }
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36'
    }
    r = requests.post(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['data'])
    temp_df.rename(columns={
        'tradeType': "-",
        'variety': "品种",
        'varietyOrder': "-",
        'contractId': "合约代码",
        'unit': "交易单位",
        'tick': "最小变动单位",
        'startTradeDate': "开始交易日",
        'endTradeDate': "最后交易日",
        'endDeliveryDate0': "最后交割日",
    }, inplace=True)
    temp_df['交易单位'] = pd.to_numeric(temp_df['交易单位'], errors="coerce")
    temp_df['最小变动单位'] = pd.to_numeric(temp_df['最小变动单位'], errors="coerce")
    temp_df['开始交易日'] = pd.to_datetime(temp_df['开始交易日'], format="%Y%m%d", errors="coerce").dt.date
    temp_df['最后交易日'] = pd.to_datetime(temp_df['最后交易日'], format="%Y%m%d", errors="coerce").dt.date
    temp_df['最后交割日'] = pd.to_datetime(temp_df['最后交割日'], format="%Y%m%d", errors="coerce").dt.date
    temp_df = temp_df[[
        "品种",
        "合约代码",
        "交易单位",
        "最小变动单位",
        "开始交易日",
        "最后交易日",
        "最后交割日",
    ]]
    return temp_df


if __name__ == '__main__':
    futures_contract_info_gfex_df = futures_contract_info_gfex()
    print(futures_contract_info_gfex_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/11/5 20:00
Desc: 基金经理大全
https://fund.eastmoney.com/manager/default.html
"""
import pandas as pd
import requests
from tqdm import tqdm

from byteapp_ak.utils import demjson


def fund_manager_em() -> pd.DataFrame:
    """
    天天基金网-基金数据-基金经理大全
    https://fund.eastmoney.com/manager/default.html
    :return: 基金经理大全
    :rtype: pandas.DataFrame
    """
    big_df = pd.DataFrame()
    url = "https://fund.eastmoney.com/Data/FundDataPortfolio_Interface.aspx"
    params = {
        "dt": "14",
        "mc": "returnjson",
        "ft": "all",
        "pn": "500",
        "pi": "1",
        "sc": "abbname",
        "st": "asc",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text.strip("var returnjson= "))
    total_page = data_json["pages"]
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({
            "pi": page,
        })
        r = requests.get(url, params=params)
        data_text = r.text
        data_json = demjson.decode(data_text.strip("var returnjson= "))
        temp_df = pd.DataFrame(data_json["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = range(1, len(big_df) + 1)
    big_df.columns = [
        "序号",
        "_",
        "姓名",
        "_",
        "所属公司",
        "_",
        "现任基金",
        "累计从业时间",
        "现任基金最佳回报",
        "_",
        "_",
        "现任基金资产总规模",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "姓名",
            "所属公司",
            "现任基金",
            "累计从业时间",
            "现任基金资产总规模",
            "现任基金最佳回报",
        ]
    ]
    big_df["现任基金最佳回报"] = big_df["现任基金最佳回报"].str.split("%", expand=True).iloc[:, 0]
    big_df["现任基金资产总规模"] = big_df["现任基金资产总规模"].str.split("亿元", expand=True).iloc[:, 0]
    big_df["累计从业时间"] = pd.to_numeric(big_df["累计从业时间"], errors="coerce")
    big_df["现任基金最佳回报"] = pd.to_numeric(big_df["现任基金最佳回报"], errors="coerce")
    big_df["现任基金资产总规模"] = pd.to_numeric(big_df["现任基金资产总规模"], errors="coerce")
    big_df["现任基金"] = big_df["现任基金"].apply(lambda x: x.split(","))
    big_df = big_df.explode(column="现任基金")
    big_df.reset_index(drop=True, inplace=True)
    return big_df


if __name__ == "__main__":
    fund_manager_em_df = fund_manager_em()
    print(fund_manager_em_df)



# !/usr/bin/env python
"""
Date: 2024/4/7 15:30
Desc: 通用帮助函数
"""

from typing import List

import pandas as pd


def set_df_columns(df: pd.DataFrame, cols: List[str]) -> pd.DataFrame:
    """
    设置 pandas.DataFrame 为空的情况
    :param df: 需要设置命名的数据框
    :type df: pandas.DataFrame
    :param cols: 字段的列表
    :type cols: list
    :return: 重新设置后的数据
    :rtype: pandas.DataFrame
    """
    if df.shape == (0, 0):
        return pd.DataFrame(data=[], columns=cols)
    else:
        df.columns = cols
        return df



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2023/10/11 15:30
Desc: 新股和风险警示股
新浪-行情中心-沪深股市-次新股
http://vip.stock.finance.sina.com.cn/mkt/#new_stock
东方财富网-行情中心-沪深个股-风险警示板
https://quote.eastmoney.com/center/gridlist.html#st_board
"""
import math

import pandas as pd
import requests


def stock_zh_a_st_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-沪深个股-风险警示板
    https://quote.eastmoney.com/center/gridlist.html#st_board
    :return: 风险警示板
    :rtype: pandas.DataFrame
    """
    url = 'http://40.push2.eastmoney.com/api/qt/clist/get'
    params = {
        'pn': '1',
        'pz': '2000',
        'po': '1',
        'np': '1',
        'ut': 'bd1d9ddb04089700cf9c27f6f7426281',
        'fltt': '2',
        'invt': '2',
        'fid': 'f3',
        'fs': 'm:0 f:4,m:1 f:4',
        'fields': 'f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,f23,f24,f25,f22,f11,f62,f128,f136,f115,f152',
        '_': '1631107510188',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['data']['diff'])
    temp_df.reset_index(inplace=True)
    temp_df['index'] = range(1, len(temp_df)+1)
    temp_df.columns = [
        '序号',
        '最新价',
        '涨跌幅',
        '涨跌额',
        '成交量',
        '成交额',
        '振幅',
        '换手率',
        '市盈率-动态',
        '量比',
        '_',
        '代码',
        '_',
        '名称',
        '最高',
        '最低',
        '今开',
        '昨收',
        '_',
        '_',
        '_',
        '市净率',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
    ]
    temp_df = temp_df[[
        '序号',
        '代码',
        '名称',
        '最新价',
        '涨跌幅',
        '涨跌额',
        '成交量',
        '成交额',
        '振幅',
        '最高',
        '最低',
        '今开',
        '昨收',
        '量比',
        '换手率',
        '市盈率-动态',
        '市净率',
    ]]
    temp_df['最新价'] = pd.to_numeric(temp_df['最新价'], errors="coerce")
    temp_df['涨跌幅'] = pd.to_numeric(temp_df['涨跌幅'], errors="coerce")
    temp_df['涨跌额'] = pd.to_numeric(temp_df['涨跌额'], errors="coerce")
    temp_df['成交量'] = pd.to_numeric(temp_df['成交量'], errors="coerce")
    temp_df['成交额'] = pd.to_numeric(temp_df['成交额'], errors="coerce")
    temp_df['振幅'] = pd.to_numeric(temp_df['振幅'], errors="coerce")
    temp_df['最高'] = pd.to_numeric(temp_df['最高'], errors="coerce")
    temp_df['最低'] = pd.to_numeric(temp_df['最低'], errors="coerce")
    temp_df['今开'] = pd.to_numeric(temp_df['今开'], errors="coerce")
    temp_df['量比'] = pd.to_numeric(temp_df['量比'], errors="coerce")
    temp_df['换手率'] = pd.to_numeric(temp_df['换手率'], errors="coerce")
    return temp_df


def stock_zh_a_new_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-沪深个股-新股
    https://quote.eastmoney.com/center/gridlist.html#newshares
    :return: 新股
    :rtype: pandas.DataFrame
    """
    url = 'http://40.push2.eastmoney.com/api/qt/clist/get'
    params = {
        'pn': '1',
        'pz': '2000',
        'po': '1',
        'np': '1',
        'ut': 'bd1d9ddb04089700cf9c27f6f7426281',
        'fltt': '2',
        'invt': '2',
        'fid': 'f26',
        'fs': 'm:0 f:8,m:1 f:8',
        'fields': 'f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,f23,f24,f25,f22,f11,f62,f128,f136,f115,f152',
        '_': '1631107510188',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['data']['diff'])
    temp_df.reset_index(inplace=True)
    temp_df['index'] = range(1, len(temp_df)+1)
    temp_df.columns = [
        '序号',
        '最新价',
        '涨跌幅',
        '涨跌额',
        '成交量',
        '成交额',
        '振幅',
        '换手率',
        '市盈率-动态',
        '量比',
        '_',
        '代码',
        '_',
        '名称',
        '最高',
        '最低',
        '今开',
        '昨收',
        '_',
        '_',
        '_',
        '市净率',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
    ]
    temp_df = temp_df[[
        '序号',
        '代码',
        '名称',
        '最新价',
        '涨跌幅',
        '涨跌额',
        '成交量',
        '成交额',
        '振幅',
        '最高',
        '最低',
        '今开',
        '昨收',
        '量比',
        '换手率',
        '市盈率-动态',
        '市净率',
    ]]
    temp_df['最新价'] = pd.to_numeric(temp_df['最新价'], errors="coerce")
    temp_df['涨跌幅'] = pd.to_numeric(temp_df['涨跌幅'], errors="coerce")
    temp_df['涨跌额'] = pd.to_numeric(temp_df['涨跌额'], errors="coerce")
    temp_df['成交量'] = pd.to_numeric(temp_df['成交量'], errors="coerce")
    temp_df['成交额'] = pd.to_numeric(temp_df['成交额'], errors="coerce")
    temp_df['振幅'] = pd.to_numeric(temp_df['振幅'], errors="coerce")
    temp_df['最高'] = pd.to_numeric(temp_df['最高'], errors="coerce")
    temp_df['最低'] = pd.to_numeric(temp_df['最低'], errors="coerce")
    temp_df['今开'] = pd.to_numeric(temp_df['今开'], errors="coerce")
    temp_df['量比'] = pd.to_numeric(temp_df['量比'], errors="coerce")
    temp_df['换手率'] = pd.to_numeric(temp_df['换手率'], errors="coerce")
    return temp_df


def stock_zh_a_stop_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-沪深个股-两网及退市
    http://quote.eastmoney.com/center/gridlist.html#staq_net_board
    :return: 两网及退市
    :rtype: pandas.DataFrame
    """
    url = 'http://40.push2.eastmoney.com/api/qt/clist/get'
    params = {
        'pn': '1',
        'pz': '2000',
        'po': '1',
        'np': '1',
        'ut': 'bd1d9ddb04089700cf9c27f6f7426281',
        'fltt': '2',
        'invt': '2',
        'fid': 'f3',
        'fs': 'm:0 s:3',
        'fields': 'f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,f23,f24,f25,f22,f11,f62,f128,f136,f115,f152',
        '_': '1631107510188',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['data']['diff'])
    temp_df.reset_index(inplace=True)
    temp_df['index'] = range(1, len(temp_df)+1)
    temp_df.columns = [
        '序号',
        '最新价',
        '涨跌幅',
        '涨跌额',
        '成交量',
        '成交额',
        '振幅',
        '换手率',
        '市盈率-动态',
        '量比',
        '_',
        '代码',
        '_',
        '名称',
        '最高',
        '最低',
        '今开',
        '昨收',
        '_',
        '_',
        '_',
        '市净率',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
    ]
    temp_df = temp_df[[
        '序号',
        '代码',
        '名称',
        '最新价',
        '涨跌幅',
        '涨跌额',
        '成交量',
        '成交额',
        '振幅',
        '最高',
        '最低',
        '今开',
        '昨收',
        '量比',
        '换手率',
        '市盈率-动态',
        '市净率',
    ]]
    temp_df['最新价'] = pd.to_numeric(temp_df['最新价'], errors="coerce")
    temp_df['涨跌幅'] = pd.to_numeric(temp_df['涨跌幅'], errors="coerce")
    temp_df['涨跌额'] = pd.to_numeric(temp_df['涨跌额'], errors="coerce")
    temp_df['成交量'] = pd.to_numeric(temp_df['成交量'], errors="coerce")
    temp_df['成交额'] = pd.to_numeric(temp_df['成交额'], errors="coerce")
    temp_df['振幅'] = pd.to_numeric(temp_df['振幅'], errors="coerce")
    temp_df['最高'] = pd.to_numeric(temp_df['最高'], errors="coerce")
    temp_df['最低'] = pd.to_numeric(temp_df['最低'], errors="coerce")
    temp_df['今开'] = pd.to_numeric(temp_df['今开'], errors="coerce")
    temp_df['量比'] = pd.to_numeric(temp_df['量比'], errors="coerce")
    temp_df['换手率'] = pd.to_numeric(temp_df['换手率'], errors="coerce")
    return temp_df


def stock_zh_a_new() -> pd.DataFrame:
    """
    新浪财经-行情中心-沪深股市-次新股
    http://vip.stock.finance.sina.com.cn/mkt/#new_stock
    :return: 次新股行情数据
    :rtype: pandas.DataFrame
    """
    url = "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeStockCount"
    params = {"node": "new_stock"}
    r = requests.get(url, params=params)
    total_page = math.ceil(int(r.json()) / 80)
    url = "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeData"
    big_df = pd.DataFrame()
    for page in range(1, total_page + 1):
        params = {
            "page": str(page),
            "num": "80",
            "sort": "symbol",
            "asc": "1",
            "node": "new_stock",
            "symbol": "",
            "_s_r_a": "page",
        }
        r = requests.get(url, params=params)
        r.encoding = "gb2312"
        data_json = r.json()
        temp_df = pd.DataFrame(data_json)
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df = big_df[
        [
            "symbol",
            "code",
            "name",
            "open",
            "high",
            "low",
            "volume",
            "amount",
            "mktcap",
            "turnoverratio",
        ]
    ]
    big_df['open'] = pd.to_numeric(big_df['open'])
    big_df['high'] = pd.to_numeric(big_df['high'])
    big_df['low'] = pd.to_numeric(big_df['low'])
    return big_df


if __name__ == "__main__":
    stock_zh_a_st_em_df = stock_zh_a_st_em()
    print(stock_zh_a_st_em_df)

    stock_zh_a_new_em_df = stock_zh_a_new_em()
    print(stock_zh_a_new_em_df)

    stock_zh_a_stop_em_df = stock_zh_a_stop_em()
    print(stock_zh_a_stop_em_df)

    stock_zh_a_new_df = stock_zh_a_new()
    print(stock_zh_a_new_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/12/27 21:11
Desc: 东方财富-行情中心-盘口异动
https://quote.eastmoney.com/changes/
"""

import pandas as pd
import requests


def stock_changes_em(symbol: str = "大笔买入") -> pd.DataFrame:
    """
    东方财富-行情中心-盘口异动
    https://quote.eastmoney.com/changes/
    :param symbol: choice of {'火箭发射', '快速反弹', '大笔买入', '封涨停板', '打开跌停板', '有大买盘',
    '竞价上涨', '高开5日线', '向上缺口', '60日新高', '60日大幅上涨', '加速下跌', '高台跳水',
    '大笔卖出', '封跌停板', '打开涨停板', '有大卖盘', '竞价下跌', '低开5日线', '向下缺口', '60日新低', '60日大幅下跌'}
    :type symbol: str
    :return: 盘口异动
    :rtype: pandas.DataFrame
    """
    url = "https://push2ex.eastmoney.com/getAllStockChanges"
    symbol_map = {
        "火箭发射": "8201",
        "快速反弹": "8202",
        "大笔买入": "8193",
        "封涨停板": "4",
        "打开跌停板": "32",
        "有大买盘": "64",
        "竞价上涨": "8207",
        "高开5日线": "8209",
        "向上缺口": "8211",
        "60日新高": "8213",
        "60日大幅上涨": "8215",
        "加速下跌": "8204",
        "高台跳水": "8203",
        "大笔卖出": "8194",
        "封跌停板": "8",
        "打开涨停板": "16",
        "有大卖盘": "128",
        "竞价下跌": "8208",
        "低开5日线": "8210",
        "向下缺口": "8212",
        "60日新低": "8214",
        "60日大幅下跌": "8216",
    }
    reversed_symbol_map = {v: k for k, v in symbol_map.items()}
    params = {
        "type": symbol_map[symbol],
        "pageindex": "0",
        "pagesize": "5000",
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "dpt": "wzchanges",
        "_": "1624005264245",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["allstock"])
    temp_df["tm"] = pd.to_datetime(temp_df["tm"], format="%H%M%S").dt.time
    temp_df.columns = [
        "时间",
        "代码",
        "_",
        "名称",
        "板块",
        "相关信息",
    ]
    temp_df = temp_df[
        [
            "时间",
            "代码",
            "名称",
            "板块",
            "相关信息",
        ]
    ]
    temp_df["板块"] = temp_df["板块"].astype(str)
    temp_df["板块"] = temp_df["板块"].map(reversed_symbol_map)
    return temp_df


def stock_board_change_em() -> pd.DataFrame:
    """
    东方财富-行情中心-当日板块异动详情
    https://quote.eastmoney.com/changes/
    :return: 当日板块异动详情页
    :rtype: pandas.DataFrame
    """
    url = "https://push2ex.eastmoney.com/getAllBKChanges"
    params = {
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "dpt": "wzchanges",
        "pageindex": "0",
        "pagesize": "5000",
        "_": "1671978840598",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    data_df = pd.DataFrame(data_json["data"]["allbk"])
    data_df.columns = [
        "-",
        "-",
        "板块名称",
        "涨跌幅",
        "主力净流入",
        "板块异动总次数",
        "ms",
        "板块具体异动类型列表及出现次数",
    ]
    data_df["板块异动最频繁个股及所属类型-买卖方向"] = [
        item["m"] for item in data_df["ms"]
    ]
    data_df["板块异动最频繁个股及所属类型-股票代码"] = [
        item["c"] for item in data_df["ms"]
    ]
    data_df["板块异动最频繁个股及所属类型-股票名称"] = [
        item["n"] for item in data_df["ms"]
    ]
    data_df["板块异动最频繁个股及所属类型-买卖方向"] = data_df[
        "板块异动最频繁个股及所属类型-买卖方向"
    ].map({0: "大笔买入", 1: "大笔卖出"})
    data_df = data_df[
        [
            "板块名称",
            "涨跌幅",
            "主力净流入",
            "板块异动总次数",
            "板块异动最频繁个股及所属类型-股票代码",
            "板块异动最频繁个股及所属类型-股票名称",
            "板块异动最频繁个股及所属类型-买卖方向",
            "板块具体异动类型列表及出现次数",
        ]
    ]
    data_df["涨跌幅"] = pd.to_numeric(data_df["涨跌幅"], errors="coerce")
    data_df["主力净流入"] = pd.to_numeric(data_df["主力净流入"], errors="coerce")
    data_df["板块异动总次数"] = pd.to_numeric(
        data_df["板块异动总次数"], errors="coerce"
    )
    return data_df


if __name__ == "__main__":
    stock_changes_em_df = stock_changes_em(symbol="大笔买入")
    print(stock_changes_em_df)

    stock_board_change_em_df = stock_board_change_em()
    print(stock_board_change_em_df)

    for item in {
        "火箭发射",
        "快速反弹",
        "大笔买入",
        "封涨停板",
        "打开跌停板",
        "有大买盘",
        "竞价上涨",
        "高开5日线",
        "向上缺口",
        "60日新高",
        "60日大幅上涨",
        "加速下跌",
        "高台跳水",
        "大笔卖出",
        "封跌停板",
        "打开涨停板",
        "有大卖盘",
        "竞价下跌",
        "低开5日线",
        "向下缺口",
        "60日新低",
        "60日大幅下跌",
    }:
        stock_changes_em_df = stock_changes_em(symbol=item)
        print(stock_changes_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/9/15 19:00
Desc: 请求网站内容的函数: 在链接失败后可重复 20 次
"""
from io import StringIO
import time
from typing import Dict

import pandas as pd
import requests


def requests_link(url: str, encoding: str = "utf-8", method: str = "get", data: Dict = None, headers: Dict = None):
    """
    利用 requests 请求网站, 爬取网站内容, 如网站链接失败, 可重复爬取 20 次
    :param url: string 网站地址
    :param encoding: string 编码类型: "utf-8", "gbk", "gb2312"
    :param method: string 访问方法: "get", "post"
    :param data: dict 上传数据: 键值对
    :param headers: dict 游览器请求头: 键值对
    :return: requests.response 爬取返回内容: response
    """
    i = 0
    while True:
        try:
            if method == "get":
                r = requests.get(url, timeout=20, headers=headers)
                r.encoding = encoding
                return r
            elif method == "post":
                r = requests.post(url, timeout=20, data=data, headers=headers)
                r.encoding = encoding
                return r
            else:
                raise ValueError("请提供正确的请求方式")
        except:
            i += 1
            print(f"第{str(i)}次链接失败, 最多尝试 20 次")
            time.sleep(5)
            if i > 20:
                return None


def pandas_read_html_link(url: str, encoding: str = "utf-8", method: str = "get", data: Dict = None, headers: Dict = None):
    """
    利用 pandas 提供的 read_html 函数来直接提取网页中的表格内容, 如网站链接失败, 可重复爬取 20 次
    :param url: string 网站地址
    :param encoding: string 编码类型: "utf-8", "gbk", "gb2312"
    :param method: string 访问方法: "get", "post"
    :param data: dict 上传数据: 键值对
    :param headers: dict 游览器请求头: 键值对
    :return: requests.response 爬取返回内容: response
    """
    i = 0
    while True:
        try:
            if method == "get":
                r = requests.get(url, timeout=20)
                r.encoding = encoding
                r = pd.read_html(StringIO(r.text), encoding=encoding)
                return r
            elif method == "post":
                r = requests.post(url, timeout=20, data=data, headers=headers)
                r.encoding = encoding
                r = pd.read_html(StringIO(r.text), encoding=encoding)
                return r
            else:
                raise ValueError("请提供正确的请求方式")
        except requests.exceptions.Timeout as e:
            i += 1
            print(f"第{str(i)}次链接失败, 最多尝试20次", e)
            time.sleep(5)
            if i > 20:
                return None



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2019/12/30 21:02
Desc:
"""
stock_em_sy_js = """
    function getCode(num) {
            var str = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
            var codes = str.split('');
            num = num || 6;
            var code = "";
            for (var i = 0; i < num; i++) {
                code += codes[Math.floor(Math.random() * 52)]
            }
            return code
        }
    """



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/2/24 15:18
Desc: 金融期权数据
http://www.sse.com.cn/assortment/options/price/
http://www.szse.cn/market/product/option/index.html
http://www.cffex.com.cn/hs300gzqq/
http://www.cffex.com.cn/zz1000gzqq/
"""
from io import BytesIO

import pandas as pd
import requests

from byteapp_ak.option.cons import (
    SH_OPTION_PAYLOAD,
    SH_OPTION_PAYLOAD_OTHER,
    SH_OPTION_URL_50,
    SH_OPTION_URL_KING_50,
    SH_OPTION_URL_300,
    SH_OPTION_URL_KING_300,
    SH_OPTION_URL_500,
    SH_OPTION_URL_KING_500,
    SH_OPTION_URL_KC_50,
    SH_OPTION_URL_KC_KING_50,
    SH_OPTION_URL_KC_50_YFD,
    SH_OPTION_URL_KING_50_YFD,
    CFFEX_OPTION_URL_300,
)


def option_finance_sse_underlying(symbol: str = "华夏科创50ETF期权") -> pd.DataFrame:
    """
    期权标的当日行情
    http://www.sse.com.cn/assortment/options/price/
    :param symbol: choice of {"华夏上证50ETF期权", "华泰柏瑞沪深300ETF期权", "南方中证500ETF期权", "华夏科创50ETF期权", "易方达科创50ETF期权"}
    :type symbol: str
    :return: 期权标的当日行情
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "华夏上证50ETF期权": SH_OPTION_URL_50,
        "华泰柏瑞沪深300ETF期权": SH_OPTION_URL_300,
        "南方中证500ETF期权": SH_OPTION_URL_500,
        "华夏科创50ETF期权": SH_OPTION_URL_KC_50,
        "易方达科创50ETF期权": SH_OPTION_URL_KC_50_YFD,
    }
    r = requests.get(symbol_map[symbol], params=SH_OPTION_PAYLOAD)
    data_json = r.json()
    raw_data = pd.DataFrame(data_json["list"])
    raw_data.at[0, 0] = "510300"
    raw_data.at[0, 8] = pd.to_datetime(
        str(data_json["date"]) + str(data_json["time"]),
        format="%Y%m%d%H%M%S",
    )
    raw_data.columns = [
        "代码",
        "名称",
        "当前价",
        "涨跌",
        "涨跌幅",
        "振幅",
        "成交量(手)",
        "成交额(万元)",
        "更新日期",
    ]
    return raw_data


def option_finance_board(
        symbol: str = "嘉实沪深300ETF期权", end_month: str = "2306"
) -> pd.DataFrame:
    """
    期权当前交易日的行情数据
    主要为三个: 华夏上证50ETF期权, 华泰柏瑞沪深300ETF期权, 嘉实沪深300ETF期权,
    沪深300股指期权, 中证1000股指期权, 上证50股指期权, 华夏科创50ETF期权, 易方达科创50ETF期权
    http://www.sse.com.cn/assortment/options/price/
    http://www.szse.cn/market/product/option/index.html
    http://www.cffex.com.cn/hs300gzqq/
    http://www.cffex.com.cn/zz1000gzqq/
    :param symbol: choice of {"华夏上证50ETF期权", "华泰柏瑞沪深300ETF期权", "南方中证500ETF期权", "华夏科创50ETF期权", "易方达科创50ETF期权", "嘉实沪深300ETF期权", "沪深300股指期权", "中证1000股指期权", "上证50股指期权"}
    :type symbol: str
    :param end_month: 2003; 2020 年 3 月到期的期权
    :type end_month: str
    :return: 当日行情
    :rtype: pandas.DataFrame
    """
    end_month = end_month[-2:]
    if symbol == "华夏上证50ETF期权":
        r = requests.get(
            SH_OPTION_URL_KING_50.format(end_month),
            params=SH_OPTION_PAYLOAD_OTHER,
        )
        data_json = r.json()
        raw_data = pd.DataFrame(data_json["list"])
        raw_data.index = [str(data_json["date"]) + str(data_json["time"])] * data_json[
            "total"
        ]
        raw_data.columns = ["合约交易代码", "当前价", "涨跌幅", "前结价", "行权价"]
        raw_data["数量"] = [data_json["total"]] * data_json["total"]
        raw_data.reset_index(inplace=True)
        raw_data.columns = ["日期", "合约交易代码", "当前价", "涨跌幅", "前结价", "行权价", "数量"]
        return raw_data
    elif symbol == "华泰柏瑞沪深300ETF期权":
        r = requests.get(
            SH_OPTION_URL_KING_300.format(end_month),
            params=SH_OPTION_PAYLOAD_OTHER,
        )
        data_json = r.json()
        raw_data = pd.DataFrame(data_json["list"])
        raw_data.index = [str(data_json["date"]) + str(data_json["time"])] * data_json[
            "total"
        ]
        raw_data.columns = ["合约交易代码", "当前价", "涨跌幅", "前结价", "行权价"]
        raw_data["数量"] = [data_json["total"]] * data_json["total"]
        raw_data.reset_index(inplace=True)
        raw_data.columns = ["日期", "合约交易代码", "当前价", "涨跌幅", "前结价", "行权价", "数量"]
        return raw_data
    elif symbol == "南方中证500ETF期权":
        r = requests.get(
            SH_OPTION_URL_KING_500.format(end_month),
            params=SH_OPTION_PAYLOAD_OTHER,
        )
        data_json = r.json()
        raw_data = pd.DataFrame(data_json["list"])
        raw_data.index = [str(data_json["date"]) + str(data_json["time"])] * data_json[
            "total"
        ]
        raw_data.columns = ["合约交易代码", "当前价", "涨跌幅", "前结价", "行权价"]
        raw_data["数量"] = [data_json["total"]] * data_json["total"]
        raw_data.reset_index(inplace=True)
        raw_data.columns = ["日期", "合约交易代码", "当前价", "涨跌幅", "前结价", "行权价", "数量"]
        return raw_data
    elif symbol == "华夏科创50ETF期权":
        r = requests.get(
            SH_OPTION_URL_KC_KING_50.format(end_month),
            params=SH_OPTION_PAYLOAD_OTHER,
        )
        data_json = r.json()
        raw_data = pd.DataFrame(data_json["list"])
        raw_data.index = [str(data_json["date"]) + str(data_json["time"])] * data_json[
            "total"
        ]
        raw_data.columns = ["合约交易代码", "当前价", "涨跌幅", "前结价", "行权价"]
        raw_data["数量"] = [data_json["total"]] * data_json["total"]
        raw_data.reset_index(inplace=True)
        raw_data.columns = ["日期", "合约交易代码", "当前价", "涨跌幅", "前结价", "行权价", "数量"]
        return raw_data
    elif symbol == "易方达科创50ETF期权":
        r = requests.get(
            SH_OPTION_URL_KING_50_YFD.format(end_month),
            params=SH_OPTION_PAYLOAD_OTHER,
        )
        data_json = r.json()
        raw_data = pd.DataFrame(data_json["list"])
        raw_data.index = [str(data_json["date"]) + str(data_json["time"])] * data_json[
            "total"
        ]
        raw_data.columns = ["合约交易代码", "当前价", "涨跌幅", "前结价", "行权价"]
        raw_data["数量"] = [data_json["total"]] * data_json["total"]
        raw_data.reset_index(inplace=True)
        raw_data.columns = ["日期", "合约交易代码", "当前价", "涨跌幅", "前结价", "行权价", "数量"]
        return raw_data
    elif symbol == "嘉实沪深300ETF期权":
        url = "http://www.szse.cn/api/report/ShowReport/data"
        params = {
            "SHOWTYPE": "JSON",
            "CATALOGID": "ysplbrb",
            "TABKEY": "tab1",
            "PAGENO": "1",
            "random": "0.10642298535346595",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        page_num = data_json[0]["metadata"]["pagecount"]
        big_df = pd.DataFrame()
        for page in range(1, page_num + 1):
            params = {
                "SHOWTYPE": "JSON",
                "CATALOGID": "ysplbrb",
                "TABKEY": "tab1",
                "PAGENO": page,
                "random": "0.10642298535346595",
            }
            r = requests.get(url, params=params)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json[0]["data"])
            big_df = pd.concat([big_df, temp_df], ignore_index=True)

        big_df.columns = [
            "合约编码",
            "合约简称",
            "标的名称",
            "类型",
            "行权价",
            "合约单位",
            "期权行权日",
            "行权交收日",
        ]
        big_df["期权行权日"] = pd.to_datetime(big_df["期权行权日"])
        big_df["end_month"] = big_df["期权行权日"].dt.month.astype(str).str.zfill(2)
        big_df = big_df[big_df["end_month"] == end_month]
        del big_df["end_month"]
        big_df.reset_index(inplace=True, drop=True)
        return big_df
    elif symbol == "沪深300股指期权":
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36'
        }
        r = requests.get(CFFEX_OPTION_URL_300, headers=headers)
        raw_df = pd.read_table(BytesIO(r.content), sep=",")
        raw_df["end_month"] = (
            raw_df["instrument"]
            .str.split("-", expand=True)
            .iloc[:, 0]
            .str.slice(
                4,
            )
        )
        raw_df = raw_df[raw_df["end_month"] == end_month]
        del raw_df["end_month"]
        raw_df.reset_index(inplace=True, drop=True)
        return raw_df
    elif symbol == "中证1000股指期权":
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36'
        }
        url = "http://www.cffex.com.cn/quote_MO.txt"
        r = requests.get(url, headers=headers)
        raw_df = pd.read_table(BytesIO(r.content), sep=",")
        raw_df["end_month"] = (
            raw_df["instrument"]
            .str.split("-", expand=True)
            .iloc[:, 0]
            .str.slice(
                4,
            )
        )
        raw_df = raw_df[raw_df["end_month"] == end_month]
        del raw_df["end_month"]
        raw_df.reset_index(inplace=True, drop=True)
        return raw_df
    elif symbol == "上证50股指期权":
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36'
        }
        url = "http://www.cffex.com.cn/quote_HO.txt"
        r = requests.get(url, headers=headers)
        raw_df = pd.read_table(BytesIO(r.content), sep=",")
        raw_df["end_month"] = (
            raw_df["instrument"]
            .str.split("-", expand=True)
            .iloc[:, 0]
            .str.slice(
                4,
            )
        )
        raw_df = raw_df[raw_df["end_month"] == end_month]
        del raw_df["end_month"]
        raw_df.reset_index(inplace=True, drop=True)
        return raw_df


if __name__ == "__main__":
    option_finance_sse_underlying_df = option_finance_sse_underlying(
        symbol="华夏科创50ETF期权"
    )
    print(option_finance_sse_underlying_df)

    option_finance_board_df = option_finance_board(
        symbol="华夏上证50ETF期权", end_month="2306"
    )
    print(option_finance_board_df)

    option_finance_board_df = option_finance_board(
        symbol="华泰柏瑞沪深300ETF期权", end_month="2306"
    )
    print(option_finance_board_df)

    option_finance_board_df = option_finance_board(
        symbol="南方中证500ETF期权", end_month="2306"
    )
    print(option_finance_board_df)

    option_finance_board_df = option_finance_board(
        symbol="华夏科创50ETF期权", end_month="2306"
    )
    print(option_finance_board_df)

    option_finance_board_df = option_finance_board(
        symbol="易方达科创50ETF期权", end_month="2306"
    )
    print(option_finance_board_df)

    option_finance_board_df = option_finance_board(
        symbol="嘉实沪深300ETF期权", end_month="2306"
    )
    print(option_finance_board_df)

    option_finance_board_df = option_finance_board(symbol="沪深300股指期权", end_month="2306")
    print(option_finance_board_df)

    option_finance_board_df = option_finance_board(
        symbol="中证1000股指期权", end_month="2306"
    )
    print(option_finance_board_df)

    option_finance_board_df = option_finance_board(symbol="上证50股指期权", end_month="2306")
    print(option_finance_board_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/10 19:00
Desc: 99 期货网-大宗商品库存数据
http://www.99qh.com/d/store.aspx
"""
from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup

from byteapp_ak.futures.cons import (
    qh_headers,
    sample_headers,
)


def futures_inventory_99(
        exchange: str = "大连商品交易所", symbol: str = "豆一"
) -> pd.DataFrame:
    """
    99 期货网-大宗商品库存数据
    http://www.99qh.com/d/store.aspx
    :param exchange: 交易所名称; choice of {"上海期货交易所", "郑州商品交易所", "大连商品交易所", "LME", "NYMEX", "CBOT", "NYBOT", "TOCOM", "上海国际能源交易中心", "OSE"}
    :type exchange: str
    :param symbol: 交易所对应的具体品种; 如：大连商品交易所的 豆一
    :type symbol: str
    :return: 大宗商品库存数据
    :rtype: pandas.DataFrame
    """
    data_code = {
        "1": [
            "1",
            "2",
            "3",
            "12",
            "32",
            "36",
            "37",
            "40",
            "42",
            "47",
            "56",
            "63",
            "69",
            "70",
            "79",
            "85",
        ],
        "2": [
            "4",
            "14",
            "29",
            "31",
            "33",
            "38",
            "44",
            "45",
            "50",
            "51",
            "52",
            "55",
            "59",
            "64",
            "66",
            "67",
            "75",
            "76",
            "81",
            "82",
            "87",
            "92",
            "95",
        ],
        "3": [
            "6",
            "7",
            "8",
            "15",
            "30",
            "34",
            "35",
            "39",
            "43",
            "53",
            "57",
            "58",
            "61",
            "62",
            "68",
            "80",
            "84",
            "86",
            "88",
            "89",
            "94",
        ],
        "4": ["9", "10", "16", "17", "18", "23", "28"],
        "5": ["11", "20", "21"],
        "6": ["13", "24", "25", "26", "27"],
        "7": ["19"],
        "8": ["22"],
        "10": ["78", "83", "90", "93"],
        "11": ["91"],
    }
    data_name = {
        "1": [
            "铜",
            "铝",
            "橡胶",
            "燃料油",
            "锌",
            "黄金",
            "螺纹钢",
            "线材",
            "铅",
            "白银",
            "石油沥青",
            "热轧卷板",
            "锡",
            "镍",
            "纸浆",
            "不锈钢",
        ],
        "2": [
            "强麦",
            "一号棉",
            "白糖",
            "PTA",
            "菜籽油",
            "早籼稻",
            "甲醇",
            "普麦",
            "玻璃",
            "油菜籽",
            "菜籽粕",
            "动力煤",
            "粳稻",
            "晚籼稻",
            "硅铁",
            "锰硅",
            "棉纱",
            "苹果",
            "红枣",
            "尿素",
            "纯碱",
            "短纤",
            "花生",
        ],
        "3": [
            "豆一",
            "豆二",
            "豆粕",
            "玉米",
            "豆油",
            "聚乙烯",
            "棕榈油",
            "聚氯乙烯",
            "焦炭",
            "焦煤",
            "铁矿石",
            "鸡蛋",
            "胶合板",
            "聚丙烯",
            "玉米淀粉",
            "乙二醇",
            "粳米",
            "苯乙烯",
            "纤维板",
            "液化石油气",
            "生猪",
        ],
        "4": ["LME铜", "LME铝", "LME镍", "LME铅", "LME锌", "LME锡", "LME铝合金"],
        "5": ["COMEX铜", "COMEX金", "COMEX银"],
        "6": ["CBOT大豆", "CBOT小麦", "CBOT玉米", "CBOT燕麦", "CBOT糙米"],
        "7": ["NYBOT2号棉"],
        "8": ["TOCOM橡胶"],
        "10": ["原油", "20号胶", "低硫燃料油", "国际铜"],
        "11": ["OSE橡胶"],
    }
    temp_out_exchange_name = {
        "1": "上海期货交易所",
        "2": "郑州商品交易所",
        "3": "大连商品交易所",
        "4": "LME",
        "5": "NYMEX",
        "6": "CBOT",
        "7": "NYBOT",
        "8": "TOCOM",
        "10": "上海国际能源交易中心",
        "11": "OSE",
    }
    exchange_map = {
        value: key for key, value in temp_out_exchange_name.items()
    }
    exchange = exchange_map[exchange]
    temp_symbol_code_map = dict(zip(data_name[exchange], data_code[exchange]))
    symbol = temp_symbol_code_map[symbol]
    out_exchange_name = {
        "1": "上海期货交易所",
        "2": "郑州商品交易所",
        "3": "大连商品交易所",
        "4": "LME",
        "5": "NYMEX",
        "6": "CBOT",
        "7": "NYBOT",
        "8": "TOCOM",
        "10": "上海国际能源交易中心",
        "11": "OSE",
    }
    name_temp_dict = {}
    code_temp_dict = {}
    for num in data_code.keys():
        name_temp_dict[out_exchange_name[num]] = dict(
            zip(data_code[num], data_name[num])
        )
        code_temp_dict[num] = dict(zip(data_code[num], data_name[num]))
    n = 10
    while n != 0:
        try:
            n -= 1
            session = requests.Session()
            url = "http://service.99qh.com/Storage/Storage.aspx"
            params = {"page": "99qh"}
            r = session.post(url, params=params, headers=sample_headers)
            cookie = r.cookies.get_dict()
            url = "http://service.99qh.com/Storage/Storage.aspx"
            params = {"page": "99qh"}
            r = requests.post(
                url, params=params, headers=sample_headers, cookies=cookie
            )
            soup = BeautifulSoup(r.text, "lxml")
            view_state = soup.find_all(attrs={"id": "__VIEWSTATE"})[0]["value"]
            even_validation = soup.find_all(attrs={"id": "__EVENTVALIDATION"})[
                0
            ]["value"]
            payload = {
                "__EVENTTARGET": "ddlExchName",
                "__EVENTARGUMENT": "",
                "__LASTFOCUS": "",
                "__VIEWSTATE": view_state,
                "__VIEWSTATEGENERATOR": "6EAC22FA",
                "__EVENTVALIDATION": even_validation,
                "ddlExchName": int(exchange),
                "ddlGoodsName": 1,
            }
            res = requests.post(
                url,
                params={"page": "99qh"},
                data=payload,
                headers=qh_headers,
                cookies=cookie,
            )
            soup = BeautifulSoup(res.text, "lxml")
            view_state = soup.find_all(attrs={"id": "__VIEWSTATE"})[0]["value"]
            even_validation = soup.find_all(attrs={"id": "__EVENTVALIDATION"})[
                0
            ]["value"]
            payload = {
                "__EVENTTARGET": "ddlGoodsName",
                "__EVENTARGUMENT": "",
                "__LASTFOCUS": "",
                "__VIEWSTATE": view_state,
                "__VIEWSTATEGENERATOR": "6EAC22FA",
                "__EVENTVALIDATION": even_validation,
                "ddlExchName": int(exchange),
                "ddlGoodsName": int(symbol),
            }
            res = requests.post(
                url,
                params=params,
                data=payload,
                headers=qh_headers,
                cookies=cookie,
            )
            data_df = pd.read_html(StringIO(res.text))[-1].T
            data_df.columns = data_df.iloc[0, :]
            data_df = data_df.iloc[1:, :]
            data_df.reset_index(inplace=True, drop=True)
            data_df.columns.name = None
            data_df["日期"] = pd.to_datetime(data_df["日期"]).dt.date
            data_df["库存"] = pd.to_numeric(data_df["库存"])
            data_df["增减"] = pd.to_numeric(data_df["增减"])
            data_df.sort_values("日期", inplace=True)
            data_df.reset_index(inplace=True, drop=True)
            return data_df
        except:
            continue


if __name__ == "__main__":
    futures_inventory_99_df = futures_inventory_99(
        exchange="郑州商品交易所", symbol="菜籽油"
    )
    print(futures_inventory_99_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/8/20 20:00
Desc: 东方财富网-数据中心-研究报告-东方财富分析师指数
https://data.eastmoney.com/invest/invest/list.html
"""

import pandas as pd
import requests
from byteapp_ak.utils.tqdm import get_tqdm
from byteapp_ak.utils.cons import headers


def stock_analyst_rank_em(year: str = "2024") -> pd.DataFrame:
    """
    东方财富网-数据中心-研究报告-东方财富分析师指数-东方财富分析师指数
    https://data.eastmoney.com/invest/invest/list.html
    :param year: 从 2015 年至今
    :type year: str
    :return: 东方财富分析师指数
    :rtype: pandas.DataFrame
    """
    url = "https://data.eastmoney.com/dataapi/invest/list"
    params = {
        "sortColumns": "YEAR_YIELD",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_ANALYST_INDEX_RANK",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f'(YEAR="{year}")',
        "distinct": "ANALYST_CODE",
        "limit": "top100",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        data_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, data_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = list(range(1, len(big_df) + 1))
    big_df.columns = [
        "序号",
        "分析师ID",
        "分析师名称",
        "更新日期",
        "年度",
        "分析师单位",
        "_",
        "年度指数",
        f"{year}年收益率",
        "3个月收益率",
        "6个月收益率",
        "12个月收益率",
        "成分股个数",
        f"{year}最新个股评级-股票名称",
        "_",
        f"{year}最新个股评级-股票代码",
        "_",
        "行业代码",
        "行业",
    ]
    big_df = big_df[
        [
            "序号",
            "分析师名称",
            "分析师单位",
            "年度指数",
            f"{year}年收益率",
            "3个月收益率",
            "6个月收益率",
            "12个月收益率",
            "成分股个数",
            f"{year}最新个股评级-股票名称",
            f"{year}最新个股评级-股票代码",
            "分析师ID",
            "行业代码",
            "行业",
            "更新日期",
            "年度",
        ]
    ]
    big_df["更新日期"] = pd.to_datetime(big_df["更新日期"], errors="coerce").dt.date
    big_df["年度指数"] = pd.to_numeric(big_df["年度指数"], errors="coerce")
    big_df[f"{year}年收益率"] = pd.to_numeric(
        big_df[f"{year}年收益率"], errors="coerce"
    )
    big_df["3个月收益率"] = pd.to_numeric(big_df["3个月收益率"], errors="coerce")
    big_df["6个月收益率"] = pd.to_numeric(big_df["6个月收益率"], errors="coerce")
    big_df["12个月收益率"] = pd.to_numeric(big_df["12个月收益率"], errors="coerce")
    big_df["成分股个数"] = pd.to_numeric(big_df["成分股个数"], errors="coerce")
    return big_df


def stock_analyst_detail_em(
    analyst_id: str = "11000200926", indicator: str = "最新跟踪成分股"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-研究报告-东方财富分析师指数-东方财富分析师指数2020最新排行-分析师详情
    https://data.eastmoney.com/invest/invest/11000257131.html
    :param analyst_id: 分析师 ID, 从 ak.stock_analyst_rank_em() 获取
    :type analyst_id: str
    :param indicator: choice of {"最新跟踪成分股", "历史跟踪成分股", "历史指数"}
    :type indicator: str
    :return: 具体指标的数据
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/special/api/data/v1/get"
    if indicator == "最新跟踪成分股":
        params = {
            "reportName": "RPT_RESEARCHER_NTCSTOCK",
            "columns": "ALL",
            "source": "WEB",
            "client": "WEB",
            "sortColumns": "CHANGE_DATE",
            "sortTypes": "-1",
            "pageNumber": "1",
            "pageSize": "1000",
            "filter": f'(ANALYST_CODE="{analyst_id}")',
            "_": "1675744438197",
        }
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        temp_df.reset_index(inplace=True)
        temp_df["index"] = list(range(1, len(temp_df) + 1))
        temp_df.columns = [
            "序号",
            "最新评级日期",
            "-",
            "-",
            "-",
            "-",
            "股票代码",
            "-",
            "股票名称",
            "调入日期",
            "当前评级名称",
            "成交价格(前复权)",
            "最新价格",
            "阶段涨跌幅",
        ]
        temp_df = temp_df[
            [
                "序号",
                "股票代码",
                "股票名称",
                "调入日期",
                "最新评级日期",
                "当前评级名称",
                "成交价格(前复权)",
                "最新价格",
                "阶段涨跌幅",
            ]
        ]
        temp_df["调入日期"] = pd.to_datetime(
            temp_df["调入日期"], errors="coerce"
        ).dt.date
        temp_df["最新评级日期"] = pd.to_datetime(
            temp_df["最新评级日期"], errors="coerce"
        ).dt.date
        temp_df["成交价格(前复权)"] = pd.to_numeric(
            temp_df["成交价格(前复权)"], errors="coerce"
        )
        temp_df["最新价格"] = pd.to_numeric(temp_df["最新价格"], errors="coerce")
        temp_df["阶段涨跌幅"] = pd.to_numeric(temp_df["阶段涨跌幅"], errors="coerce")
        return temp_df
    elif indicator == "历史跟踪成分股":
        params = {
            "reportName": "RPT_RESEARCHER_HISTORYSTOCK",
            "columns": "ALL",
            "source": "WEB",
            "client": "WEB",
            "sortColumns": "CHANGE_DATE",
            "sortTypes": "-1",
            "pageNumber": "1",
            "pageSize": "1000",
            "filter": f'(ANALYST_CODE="{analyst_id}")',
            "_": "1675744438197",
        }
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        temp_df.reset_index(inplace=True)
        temp_df["index"] = list(range(1, len(temp_df) + 1))
        temp_df.columns = [
            "序号",
            "-",
            "-",
            "-",
            "股票代码",
            "-",
            "股票名称",
            "调入日期",
            "调出日期",
            "调入时评级名称",
            "调出原因",
            "累计涨跌幅",
        ]
        temp_df = temp_df[
            [
                "序号",
                "股票代码",
                "股票名称",
                "调入日期",
                "调出日期",
                "调入时评级名称",
                "调出原因",
                "累计涨跌幅",
            ]
        ]
        temp_df["调入日期"] = pd.to_datetime(
            temp_df["调入日期"], errors="coerce"
        ).dt.date
        temp_df["调出日期"] = pd.to_datetime(
            temp_df["调出日期"], errors="coerce"
        ).dt.date
        temp_df["累计涨跌幅"] = pd.to_numeric(temp_df["累计涨跌幅"], errors="coerce")
        return temp_df
    elif indicator == "历史指数":
        params = {
            "reportName": "RPT_RESEARCHER_DETAILS",
            "columns": "ALL",
            "sortColumns": "TRADE_DATE",
            "sortTypes": "-1",
            "filter": f'(ANALYST_CODE="{analyst_id}")',
            "source": "WEB",
            "client": "WEB",
            "_": "1675744438200",
        }
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        temp_df = temp_df[
            [
                "TRADE_DATE",
                "INDEX_HVALUE",
            ]
        ]
        temp_df.columns = ["date", "value"]
        temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
        temp_df["value"] = pd.to_numeric(temp_df["value"], errors="coerce")
        temp_df.sort_values(["date"], inplace=True, ignore_index=True)
        return temp_df


if __name__ == "__main__":
    stock_analyst_rank_em_df = stock_analyst_rank_em(year="2024")
    print(stock_analyst_rank_em_df)

    stock_analyst_detail_em_df = stock_analyst_detail_em(
        analyst_id="11000200926", indicator="最新跟踪成分股"
    )
    print(stock_analyst_detail_em_df)

    stock_analyst_detail_em_df = stock_analyst_detail_em(
        analyst_id="11000200926", indicator="历史跟踪成分股"
    )
    print(stock_analyst_detail_em_df)

    stock_analyst_detail_em_df = stock_analyst_detail_em(
        analyst_id="11000200926", indicator="历史指数"
    )
    print(stock_analyst_detail_em_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2023/10/19 16:00
Desc: 50 ETF 期权波动率指数 QVIX
300 ETF 期权波动率指数 QVIX
http://1.optbbs.com/s/vix.shtml?50ETF
http://1.optbbs.com/s/vix.shtml?300ETF
"""
import pandas as pd


def index_option_50etf_qvix() -> pd.DataFrame:
    """
    50ETF 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?50ETF
    :return: 50ETF 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/k.csv"
    temp_df = pd.read_csv(url).iloc[:, :5]
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
    ]
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df['open'] = pd.to_numeric(temp_df['open'], errors="coerce")
    temp_df['high'] = pd.to_numeric(temp_df['high'], errors="coerce")
    temp_df['low'] = pd.to_numeric(temp_df['low'], errors="coerce")
    temp_df['close'] = pd.to_numeric(temp_df['close'], errors="coerce")
    return temp_df


def index_option_50etf_min_qvix() -> pd.DataFrame:
    """
    50 ETF 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?50ETF
    :return: 50 ETF 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/vix50.csv"
    temp_df = pd.read_csv(url).iloc[:, :2]
    temp_df.columns = [
        "time",
        "qvix",
    ]
    temp_df['qvix'] = pd.to_numeric(temp_df['qvix'], errors="coerce")
    return temp_df


def index_option_300etf_qvix() -> pd.DataFrame:
    """
    300 ETF 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?300ETF
    :return: 300 ETF 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/k.csv"
    temp_df = pd.read_csv(url).iloc[:, [0, 9, 10, 11, 12]]
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
    ]
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df['open'] = pd.to_numeric(temp_df['open'], errors="coerce")
    temp_df['high'] = pd.to_numeric(temp_df['high'], errors="coerce")
    temp_df['low'] = pd.to_numeric(temp_df['low'], errors="coerce")
    temp_df['close'] = pd.to_numeric(temp_df['close'], errors="coerce")
    return temp_df


def index_option_300etf_min_qvix() -> pd.DataFrame:
    """
    300 ETF 期权波动率指数 QVIX-分时
    http://1.optbbs.com/s/vix.shtml?300ETF
    :return: 300 ETF 期权波动率指数 QVIX-分时
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/vix300.csv"
    temp_df = pd.read_csv(url).iloc[:, :2]
    temp_df.columns = [
        "time",
        "qvix",
    ]
    temp_df['qvix'] = pd.to_numeric(temp_df['qvix'], errors="coerce")
    return temp_df


if __name__ == "__main__":
    index_option_50etf_qvix_df = index_option_50etf_qvix()
    print(index_option_50etf_qvix_df)

    index_option_50etf_min_qvix_df = index_option_50etf_min_qvix()
    print(index_option_50etf_min_qvix_df)

    index_option_300etf_qvix_df = index_option_300etf_qvix()
    print(index_option_300etf_qvix_df)

    index_option_300etf_min_qvix_df = index_option_300etf_min_qvix()
    print(index_option_300etf_min_qvix_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/5/27 20:19
Desc: 指数配置文件
"""
# weibo-user-agent
index_weibo_headers = {
    "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 Mobile/15A372 Safari/604.1",
    "Referer": "http://data.weibo.com/index/newindex",
    "Accept": "application/json",
    "Origin": "https://data.weibo.com",
}

# sw-cons
sw_cons_headers = {
    "Accept": "*/*",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Connection": "keep-alive",
    "Content-Length": "34",
    "Content-Type": "text/plain; charset=UTF-8",
    # "Cookie": "ASP.NET_SessionId=i55eaz55142xdxfx0bkqp145",
    "Host": "www.swsindex.com",
    "Origin": "http://www.swsindex.com",
    "Referer": "http://www.swsindex.com/idx0210.aspx?swindexcode=801010",
    'User-Agent': "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36",
    "X-AjaxPro-Method": "ReturnContent",
}

# sw-url
sw_url = "http://www.swsindex.com/handler.aspx"

# sw-payload
sw_payload = {
    "tablename": "swzs",
    "key": "L1",
    "p": "1",
    "where": "L1 in('801010','801020','801030','801040','801050','801080','801110','801120','801130','801140','801150','801160','801170','801180','801200','801210','801230','801710','801720','801730','801740','801750','801760','801770','801780','801790','801880','801890','801950','801960','801970','801980')",
    "orderby": "",
    "fieldlist": "L1,L2,L3,L4,L5,L6,L7,L8,L11",
    "pagecount": "28",
    "timed": "",
}

# sw-headers
sw_headers = {
    'Accept': 'application/json, text/javascript, */*',
    'Accept-Encoding': 'gzip, deflate',
    'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8',
    'Cache-Control': 'no-cache',
    'Content-Type': 'application/x-www-form-urlencoded',
    'DNT': '1',
    'Host': 'www.swsindex.com',
    'Origin': 'http://www.swsindex.com',
    'Pragma': 'no-cache',
    'Referer': 'http://www.swsindex.com/idx0120.aspx?columnid=8832',
    'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.169 Safari/537.36',
    'X-Requested-With': 'XMLHttpRequest'
}

# zh-sina-a
zh_sina_index_stock_url = "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeDataSimple"
zh_sina_index_stock_payload = {
    "page": "1",
    "num": "80",
    "sort": "symbol",
    "asc": "1",
    "node": "hs_s",
    "_s_r_a": "page"
}
zh_sina_index_stock_count_url = "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeStockCountSimple?node=hs_s"
zh_sina_index_stock_hist_url = "https://finance.sina.com.cn/realstock/company/{}/hisdata/klc_kl.js"

# investing
short_headers = {
    'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.91 Safari/537.36'
}

long_headers = {
    'accept': 'text/plain, */*; q=0.01',
    # 'accept-encoding': 'gzip, deflate, br',
    'accept-language': 'zh-CN,zh;q=0.9,en;q=0.8',
    # 'cache-control': 'no-cache',
    'content-length': '267',
    'content-type': 'application/x-www-form-urlencoded',
    'origin': 'https://cn.investing.com',
    'referer': 'https://cn.investing.com/commodities/brent-oil-historical-data',
    'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.91 Safari/537.36',
    'x-requested-with': 'XMLHttpRequest',
}
